# Copyright 2015 IBM Corp.
# Copyright 2017 AT&T Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock


from tempest.lib.services.compute import base_compute_client
from tempest.lib.services.compute import servers_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestServersClient(base.BaseServiceTest):

    FAKE_SERVERS = {'servers': [{
        "id": "616fb98f-46ca-475e-917e-2563e5a8cd19",
        "links": [
            {
                "href": "http://os.co/v2/616fb98f-46ca-475e-917e-2563e5a8cd19",
                "rel": "self"
            },
            {
                "href": "http://os.co/616fb98f-46ca-475e-917e-2563e5a8cd19",
                "rel": "bookmark"
            }
        ],
        "name": u"new\u1234-server-test"}]
    }

    FAKE_SERVER_DIAGNOSTICS = {
        "cpu0_time": 17300000000,
        "memory": 524288,
        "vda_errors": -1,
        "vda_read": 262144,
        "vda_read_req": 112,
        "vda_write": 5778432,
        "vda_write_req": 488,
        "vnet1_rx": 2070139,
        "vnet1_rx_drop": 0,
        "vnet1_rx_errors": 0,
        "vnet1_rx_packets": 26701,
        "vnet1_tx": 140208,
        "vnet1_tx_drop": 0,
        "vnet1_tx_errors": 0,
        "vnet1_tx_packets": 662
    }

    FAKE_SERVER_GET = {'server': {
        "accessIPv4": "",
        "accessIPv6": "",
        "addresses": {
            "private": [
                {
                    "addr": "192.168.0.3",
                    "version": 4
                }
            ]
        },
        "created": "2012-08-20T21:11:09Z",
        "flavor": {
            "id": "1",
            "links": [
                {
                    "href": "http://os.com/openstack/flavors/1",
                    "rel": "bookmark"
                }
            ]
        },
        "hostId": "65201c14a29663e06d0748e561207d998b343e1d164bfa0aafa9c45d",
        "id": "893c7791-f1df-4c3d-8383-3caae9656c62",
        "image": {
            "id": "70a599e0-31e7-49b7-b260-868f441e862b",
            "links": [
                {
                    "href": "http://imgs/70a599e0-31e7-49b7-b260-868f441e862b",
                    "rel": "bookmark"
                }
            ]
        },
        "links": [
            {
                "href": "http://v2/srvs/893c7791-f1df-4c3d-8383-3caae9656c62",
                "rel": "self"
            },
            {
                "href": "http://srvs/893c7791-f1df-4c3d-8383-3caae9656c62",
                "rel": "bookmark"
            }
        ],
        "metadata": {
            u"My Server N\u1234me": u"Apa\u1234che1"
        },
        "name": u"new\u1234-server-test",
        "progress": 0,
        "status": "ACTIVE",
        "tenant_id": "openstack",
        "updated": "2012-08-20T21:11:09Z",
        "user_id": "fake"}
    }

    FAKE_SERVER_POST = {"server": {
        "id": "616fb98f-46ca-475e-917e-2563e5a8cd19",
        "adminPass": "fake-admin-pass",
        "security_groups": [
            'fake-security-group-1',
            'fake-security-group-2'
        ],
        "links": [
            {
                "href": "http://os.co/v2/616fb98f-46ca-475e-917e-2563e5a8cd19",
                "rel": "self"
            },
            {
                "href": "http://os.co/616fb98f-46ca-475e-917e-2563e5a8cd19",
                "rel": "bookmark"
            }
        ],
        "OS-DCF:diskConfig": "fake-disk-config"}
    }

    FAKE_ADDRESS = {"addresses": {
        "private": [
            {
                "addr": "192.168.0.3",
                "version": 4
            }
        ]}
    }

    FAKE_COMMON_VOLUME = {
        "id": "a6b0875b-6b5d-4a5a-81eb-0c3aa62e5fdb",
        "device": "fake-device",
        "volumeId": "a6b0875b-46ca-475e-917e-0c3aa62e5fdb",
        "serverId": "616fb98f-46ca-475e-917e-2563e5a8cd19"
    }

    FAKE_VIRTUAL_INTERFACES = {
        "id": "a6b0875b-46ca-475e-917e-0c3aa62e5fdb",
        "mac_address": "00:25:90:5b:f8:c3",
        "OS-EXT-VIF-NET:net_id": "fake-os-net-id"
    }

    FAKE_INSTANCE_ACTIONS = {
        "action": "fake-action",
        "request_id": "16fb98f-46ca-475e-917e-2563e5a8cd19",
        "user_id": "16fb98f-46ca-475e-917e-2563e5a8cd12",
        "project_id": "16fb98f-46ca-475e-917e-2563e5a8cd34",
        "start_time": "2016-10-02T10:00:00-05:00",
        "message": "fake-msg",
        "instance_uuid": "16fb98f-46ca-475e-917e-2563e5a8cd12"
    }

    FAKE_VNC_CONSOLE = {
        "type": "fake-type",
        "url": "http://os.co/v2/616fb98f-46ca-475e-917e-2563e5a8cd19"
    }

    FAKE_SERVER_PASSWORD = {
        "adminPass": "fake-password",
    }

    FAKE_INSTANCE_ACTION_EVENTS = {
        "event": "fake-event",
        "start_time": "2016-10-02T10:00:00-05:00",
        "finish_time": "2016-10-02T10:00:00-05:00",
        "result": "fake-result",
        "traceback": "fake-trace-back"
    }

    FAKE_SECURITY_GROUPS = [{
        "description": "default",
        "id": "3fb26eb3-581b-4420-9963-b0879a026506",
        "name": "default",
        "rules": [],
        "tenant_id": "openstack"
    }]

    FAKE_INSTANCE_WITH_EVENTS = copy.deepcopy(FAKE_INSTANCE_ACTIONS)
    FAKE_INSTANCE_WITH_EVENTS['events'] = [FAKE_INSTANCE_ACTION_EVENTS]

    FAKE_REBUILD_SERVER = copy.deepcopy(FAKE_SERVER_GET)
    FAKE_REBUILD_SERVER['server']['adminPass'] = 'fake-admin-pass'

    FAKE_TAGS = ["foo", "bar"]
    REPLACE_FAKE_TAGS = ["baz", "qux"]

    server_id = FAKE_SERVER_GET['server']['id']
    network_id = 'a6b0875b-6b5d-4a5a-81eb-0c3aa62e5fdb'

    def setUp(self):
        super(TestServersClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = servers_client.ServersClient(
            fake_auth, 'compute', 'regionOne')
        self.addCleanup(mock.patch.stopall)

    def test_list_servers_with_str_body(self):
        self._test_list_servers()

    def test_list_servers_with_bytes_body(self):
        self._test_list_servers(bytes_body=True)

    def _test_list_servers(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_servers,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVERS,
            bytes_body)

    def test_show_server_with_str_body(self):
        self._test_show_server()

    def test_show_server_with_bytes_body(self):
        self._test_show_server(bytes_body=True)

    def _test_show_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_server,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVER_GET,
            bytes_body,
            server_id=self.server_id
            )

    def test_delete_server(self):
        self.check_service_client_function(
            self.client.delete_server,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            server_id=self.server_id
            )

    def test_create_server_with_str_body(self):
        self._test_create_server()

    def test_create_server_with_bytes_body(self):
        self._test_create_server(True)

    def _test_create_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_server,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_SERVER_POST,
            bytes_body,
            status=202,
            name='fake-name',
            imageRef='fake-image-ref',
            flavorRef='fake-flavor-ref'
            )

    def test_list_addresses_with_str_body(self):
        self._test_list_addresses()

    def test_list_addresses_with_bytes_body(self):
        self._test_list_addresses(True)

    def _test_list_addresses(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_addresses,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ADDRESS,
            bytes_body,
            server_id=self.server_id
            )

    def test_list_addresses_by_network_with_str_body(self):
        self._test_list_addresses_by_network()

    def test_list_addresses_by_network_with_bytes_body(self):
        self._test_list_addresses_by_network(True)

    def _test_list_addresses_by_network(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_addresses_by_network,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ADDRESS['addresses'],
            bytes_body,
            server_id=self.server_id,
            network_id=self.network_id
            )

    def test_action_with_str_body(self):
        self._test_action()

    def test_action_with_bytes_body(self):
        self._test_action(True)

    def _test_action(self, bytes_body=False):
        self.check_service_client_function(
            self.client.action,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            bytes_body,
            server_id=self.server_id,
            action_name='fake-action-name',
            schema={'status_code': 200}
            )

    def test_create_backup_with_str_body(self):
        self._test_create_backup()

    def test_create_backup_with_bytes_body(self):
        self._test_create_backup(True)

    def _test_create_backup(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_backup,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            bytes_body,
            status=202,
            server_id=self.server_id,
            backup_type='fake-backup',
            rotation='fake-rotation',
            name='fake-name'
            )

    def test_evacuate_server_with_str_body(self):
        self._test_evacuate_server()

    def test_evacuate_server_with_bytes_body(self):
        self._test_evacuate_server(bytes_body=True)

    def _test_evacuate_server(self, bytes_body=False):
        kwargs = {'server_id': self.server_id,
                  'host': 'fake-target-host'}
        self.check_service_client_function(
            self.client.evacuate_server,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_SERVER_PASSWORD,
            bytes_body,
            **kwargs)

    def test_change_password_with_str_body(self):
        self._test_change_password()

    def test_change_password_with_bytes_body(self):
        self._test_change_password(True)

    def _test_change_password(self, bytes_body=False):
        self.check_service_client_function(
            self.client.change_password,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            bytes_body,
            status=202,
            server_id=self.server_id,
            adminPass='fake-admin-pass'
            )

    def test_show_password_with_str_body(self):
        self._test_show_password()

    def test_show_password_with_bytes_body(self):
        self._test_show_password(True)

    def _test_show_password(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_password,
            'tempest.lib.common.rest_client.RestClient.get',
            {'password': 'fake-password'},
            bytes_body,
            server_id=self.server_id
            )

    def test_delete_password(self):
        self.check_service_client_function(
            self.client.delete_password,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            server_id=self.server_id
            )

    def test_reboot_server(self):
        self.check_service_client_function(
            self.client.reboot_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id,
            type='fake-reboot-type'
            )

    def test_rebuild_server_with_str_body(self):
        self._test_rebuild_server()

    def test_rebuild_server_with_bytes_body(self):
        self._test_rebuild_server(True)

    def _test_rebuild_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.rebuild_server,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_REBUILD_SERVER,
            bytes_body,
            status=202,
            server_id=self.server_id,
            image_ref='fake-image-ref'
            )

    def test_resize_server(self):
        self.check_service_client_function(
            self.client.resize_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id,
            flavor_ref='fake-flavor-ref'
            )

    def test_confirm_resize_server(self):
        self.check_service_client_function(
            self.client.confirm_resize_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=204,
            server_id=self.server_id
            )

    def test_revert_resize(self):
        self.check_service_client_function(
            self.client.revert_resize_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_list_server_metadata_with_str_body(self):
        self._test_list_server_metadata()

    def test_list_server_metadata_with_bytes_body(self):
        self._test_list_server_metadata()

    def _test_list_server_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_server_metadata,
            'tempest.lib.common.rest_client.RestClient.get',
            {'metadata': {'fake-key': 'fake-meta-data'}},
            bytes_body,
            server_id=self.server_id
            )

    def test_set_server_metadata_with_str_body(self):
        self._test_set_server_metadata()

    def test_set_server_metadata_with_bytes_body(self):
        self._test_set_server_metadata(True)

    def _test_set_server_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.set_server_metadata,
            'tempest.lib.common.rest_client.RestClient.put',
            {'metadata': {'fake-key': 'fake-meta-data'}},
            bytes_body,
            server_id=self.server_id,
            meta='fake-meta'
            )

    def test_update_server_metadata_with_str_body(self):
        self._test_update_server_metadata()

    def test_update_server_metadata_with_bytes_body(self):
        self._test_update_server_metadata(True)

    def _test_update_server_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_server_metadata,
            'tempest.lib.common.rest_client.RestClient.post',
            {'metadata': {'fake-key': 'fake-meta-data'}},
            bytes_body,
            server_id=self.server_id,
            meta='fake-meta'
            )

    def test_show_server_metadata_item_with_str_body(self):
        self._test_show_server_metadata()

    def test_show_server_metadata_item_with_bytes_body(self):
        self._test_show_server_metadata(True)

    def _test_show_server_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_server_metadata_item,
            'tempest.lib.common.rest_client.RestClient.get',
            {'meta': {'fake-key': 'fake-meta-data'}},
            bytes_body,
            server_id=self.server_id,
            key='fake-key'
            )

    def test_set_server_metadata_item_with_str_body(self):
        self._test_set_server_metadata_item()

    def test_set_server_metadata_item_with_bytes_body(self):
        self._test_set_server_metadata_item(True)

    def _test_set_server_metadata_item(self, bytes_body=False):
        self.check_service_client_function(
            self.client.set_server_metadata_item,
            'tempest.lib.common.rest_client.RestClient.put',
            {'meta': {'fake-key': 'fake-meta-data'}},
            bytes_body,
            server_id=self.server_id,
            key='fake-key',
            meta='fake-meta'
            )

    def test_delete_server_metadata(self):
        self.check_service_client_function(
            self.client.delete_server_metadata_item,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            server_id=self.server_id,
            key='fake-key'
            )

    def test_stop_server(self):
        self.check_service_client_function(
            self.client.stop_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_start_server(self):
        self.check_service_client_function(
            self.client.start_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_attach_volume_with_str_body(self):
        self._test_attach_volume_server()

    def test_attach_volume_with_bytes_body(self):
        self._test_attach_volume_server(True)

    def _test_attach_volume_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.attach_volume,
            'tempest.lib.common.rest_client.RestClient.post',
            {'volumeAttachment': self.FAKE_COMMON_VOLUME},
            bytes_body,
            server_id=self.server_id
            )

    def test_update_attached_volume(self):
        self.check_service_client_function(
            self.client.update_attached_volume,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            status=202,
            server_id=self.server_id,
            attachment_id='fake-attachment-id',
            volumeId='fake-volume-id'
            )

    def test_detach_volume_with_str_body(self):
        self._test_detach_volume_server()

    def test_detach_volume_with_bytes_body(self):
        self._test_detach_volume_server(True)

    def _test_detach_volume_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.detach_volume,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            bytes_body,
            status=202,
            server_id=self.server_id,
            volume_id=self.FAKE_COMMON_VOLUME['volumeId']
            )

    def test_show_volume_attachment_with_str_body(self):
        self._test_show_volume_attachment()

    def test_show_volume_attachment_with_bytes_body(self):
        self._test_show_volume_attachment(True)

    def _test_show_volume_attachment(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_volume_attachment,
            'tempest.lib.common.rest_client.RestClient.get',
            {'volumeAttachment': self.FAKE_COMMON_VOLUME},
            bytes_body,
            server_id=self.server_id,
            volume_id=self.FAKE_COMMON_VOLUME['volumeId']
            )

    def test_list_volume_attachments_with_str_body(self):
        self._test_list_volume_attachments()

    def test_list_volume_attachments_with_bytes_body(self):
        self._test_list_volume_attachments(True)

    def _test_list_volume_attachments(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_volume_attachments,
            'tempest.lib.common.rest_client.RestClient.get',
            {'volumeAttachments': [self.FAKE_COMMON_VOLUME]},
            bytes_body,
            server_id=self.server_id
            )

    def test_add_security_group_with_str_body(self):
        self._test_add_security_group()

    def test_add_security_group_with_bytes_body(self):
        self._test_add_security_group(True)

    def _test_add_security_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.add_security_group,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            bytes_body,
            status=202,
            server_id=self.server_id,
            name='fake-name'
            )

    def test_remove_security_group(self):
        self.check_service_client_function(
            self.client.remove_security_group,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id,
            name='fake-name'
            )

    def test_live_migrate_server_with_str_body(self):
        self._test_live_migrate_server()

    def test_live_migrate_server_with_bytes_body(self):
        self._test_live_migrate_server(True)

    def _test_live_migrate_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.live_migrate_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            bytes_body,
            status=202,
            server_id=self.server_id
            )

    def test_migrate_server_with_str_body(self):
        self._test_migrate_server()

    def test_migrate_server_with_bytes_body(self):
        self._test_migrate_server(True)

    def _test_migrate_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.migrate_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            bytes_body,
            status=202,
            server_id=self.server_id
            )

    def test_lock_server(self):
        self.check_service_client_function(
            self.client.lock_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_unlock_server(self):
        self.check_service_client_function(
            self.client.unlock_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_suspend_server(self):
        self.check_service_client_function(
            self.client.suspend_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_resume_server(self):
        self.check_service_client_function(
            self.client.resume_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_pause_server(self):
        self.check_service_client_function(
            self.client.pause_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_unpause_server(self):
        self.check_service_client_function(
            self.client.unpause_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_reset_state(self):
        self.check_service_client_function(
            self.client.reset_state,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id,
            state='fake-state'
            )

    def test_shelve_server(self):
        self.check_service_client_function(
            self.client.shelve_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_unshelve_server(self):
        self.check_service_client_function(
            self.client.unshelve_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_shelve_offload_server(self):
        self.check_service_client_function(
            self.client.shelve_offload_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_get_console_output_with_str_body(self):
        self._test_get_console_output()

    def test_get_console_output_with_bytes_body(self):
        self._test_get_console_output(True)

    def _test_get_console_output(self, bytes_body=False):
        self.check_service_client_function(
            self.client.get_console_output,
            'tempest.lib.common.rest_client.RestClient.post',
            {'output': 'fake-output'},
            bytes_body,
            server_id=self.server_id,
            length='fake-length'
            )

    def test_rescue_server_with_str_body(self):
        self._test_rescue_server()

    def test_rescue_server_with_bytes_body(self):
        self._test_rescue_server(True)

    def _test_rescue_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.rescue_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {'adminPass': 'fake-admin-pass'},
            bytes_body,
            server_id=self.server_id
            )

    def test_unrescue_server(self):
        self.check_service_client_function(
            self.client.unrescue_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_show_server_diagnostics_with_str_body(self):
        self._test_show_server_diagnostics()

    def test_show_server_diagnostics_with_bytes_body(self):
        self._test_show_server_diagnostics(True)

    def _test_show_server_diagnostics(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_server_diagnostics,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVER_DIAGNOSTICS,
            bytes_body,
            status=200,
            server_id=self.server_id
            )

    def test_list_instance_actions_with_str_body(self):
        self._test_list_instance_actions()

    def test_list_instance_actions_with_bytes_body(self):
        self._test_list_instance_actions(True)

    def _test_list_instance_actions(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_instance_actions,
            'tempest.lib.common.rest_client.RestClient.get',
            {'instanceActions': [self.FAKE_INSTANCE_ACTIONS]},
            bytes_body,
            server_id=self.server_id
            )

    def test_show_instance_action_with_str_body(self):
        self._test_show_instance_action()

    def test_show_instance_action_with_bytes_body(self):
        self._test_show_instance_action(True)

    def _test_show_instance_action(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_instance_action,
            'tempest.lib.common.rest_client.RestClient.get',
            {'instanceAction': self.FAKE_INSTANCE_WITH_EVENTS},
            bytes_body,
            server_id=self.server_id,
            request_id='fake-request-id'
            )

    def test_force_delete_server(self):
        self.check_service_client_function(
            self.client.force_delete_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_restore_soft_deleted_server(self):
        self.check_service_client_function(
            self.client.restore_soft_deleted_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_reset_network(self):
        self.check_service_client_function(
            self.client.reset_network,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_inject_network_info(self):
        self.check_service_client_function(
            self.client.inject_network_info,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            server_id=self.server_id
            )

    def test_get_vnc_console_with_str_body(self):
        self._test_get_vnc_console()

    def test_get_vnc_console_with_bytes_body(self):
        self._test_get_vnc_console(True)

    def _test_get_vnc_console(self, bytes_body=False):
        self.check_service_client_function(
            self.client.get_vnc_console,
            'tempest.lib.common.rest_client.RestClient.post',
            {'console': self.FAKE_VNC_CONSOLE},
            bytes_body,
            server_id=self.server_id,
            type='fake-console-type'
            )

    def test_list_security_groups_by_server_with_str_body(self):
        self._test_list_security_groups_by_server()

    def test_list_security_groups_by_server_with_bytes_body(self):
        self._test_list_security_groups_by_server(True)

    def _test_list_security_groups_by_server(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_security_groups_by_server,
            'tempest.lib.common.rest_client.RestClient.get',
            {'security_groups': self.FAKE_SECURITY_GROUPS},
            bytes_body,
            server_id=self.server_id
            )

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_list_tags_str_body(self, _):
        self._test_list_tags()

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_list_tags_byte_body(self, _):
        self._test_list_tags(bytes_body=True)

    def _test_list_tags(self, bytes_body=False):
        expected = {"tags": self.FAKE_TAGS}
        self.check_service_client_function(
            self.client.list_tags,
            'tempest.lib.common.rest_client.RestClient.get',
            expected,
            server_id=self.server_id,
            to_utf=bytes_body)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_update_all_tags_str_body(self, _):
        self._test_update_all_tags()

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_update_all_tags_byte_body(self, _):
        self._test_update_all_tags(bytes_body=True)

    def _test_update_all_tags(self, bytes_body=False):
        expected = {"tags": self.REPLACE_FAKE_TAGS}
        self.check_service_client_function(
            self.client.update_all_tags,
            'tempest.lib.common.rest_client.RestClient.put',
            expected,
            server_id=self.server_id,
            tags=self.REPLACE_FAKE_TAGS,
            to_utf=bytes_body)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_delete_all_tags(self, _):
        self.check_service_client_function(
            self.client.delete_all_tags,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            server_id=self.server_id,
            status=204)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_check_tag_existence_str_body(self, _):
        self._test_check_tag_existence()

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_check_tag_existence_byte_body(self, _):
        self._test_check_tag_existence(bytes_body=True)

    def _test_check_tag_existence(self, bytes_body=False):
        self.check_service_client_function(
            self.client.check_tag_existence,
            'tempest.lib.common.rest_client.RestClient.get',
            {},
            server_id=self.server_id,
            tag=self.FAKE_TAGS[0],
            status=204,
            to_utf=bytes_body)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_update_tag_str_body(self, _):
        self._test_update_tag()

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_update_tag_byte_body(self, _):
        self._test_update_tag(bytes_body=True)

    def _test_update_tag(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_tag,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            server_id=self.server_id,
            tag=self.FAKE_TAGS[0],
            status=201,
            headers={'location': 'fake_location'},
            to_utf=bytes_body)

    @mock.patch.object(base_compute_client, 'COMPUTE_MICROVERSION',
                       new_callable=mock.PropertyMock(return_value='2.26'))
    def test_delete_tag(self, _):
        self.check_service_client_function(
            self.client.delete_tag,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            server_id=self.server_id,
            tag=self.FAKE_TAGS[0],
            status=204,
            )


class TestServersClientMinV26(base.BaseServiceTest):

    def setUp(self):
        super(TestServersClientMinV26, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = servers_client.ServersClient(fake_auth, 'compute',
                                                   'regionOne')
        base_compute_client.COMPUTE_MICROVERSION = '2.6'
        self.server_id = "920eaac8-a284-4fd1-9c2c-b30f0181b125"

    def tearDown(self):
        super(TestServersClientMinV26, self).tearDown()
        base_compute_client.COMPUTE_MICROVERSION = None

    def test_get_remote_consoles(self):
        self.check_service_client_function(
            self.client.get_remote_console,
            'tempest.lib.common.rest_client.RestClient.post',
            {
                'remote_console': {
                    'protocol': 'serial',
                    'type': 'serial',
                    'url': 'ws://127.0.0.1:6083/?token=IllAllowIt'
                    }
            },
            server_id=self.server_id,
            console_type='serial',
            protocol='serial',
            )



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class VolumesCloneNegativeTest(base.BaseVolumeTest):
    """Negative tests of volume clone"""

    @classmethod
    def skip_checks(cls):
        super(VolumesCloneNegativeTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.clone:
            raise cls.skipException("Cinder volume clones are disabled")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9adae371-a257-43a5-459a-dc7c88e66e0e')
    def test_create_from_volume_decreasing_size(self):
        """Test cloning a volume with decreasing size will fail"""
        # Creates a volume from another volume passing a size different from
        # the source volume.
        src_size = CONF.volume.volume_size + CONF.volume.volume_size_extend
        src_vol = self.create_volume(size=src_size)

        # Destination volume smaller than source
        self.assertRaises(exceptions.BadRequest,
                          self.volumes_client.create_volume,
                          size=src_size - CONF.volume.volume_size_extend,
                          source_volid=src_vol['id'])



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import extensions_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestExtensionsClient(base.BaseServiceTest):

    FAKE_EXTENSIONS = {
        "extensions": [
            {
                "updated": "2013-01-20T00:00:00-00:00",
                "name": "Neutron Service Type Management",
                "links": [],
                "alias": "service-type",
                "description": "API for retrieving service providers for"
                " Neutron advanced services"
            },
            {
                "updated": "2012-10-05T10:00:00-00:00",
                "name": "security-group",
                "links": [],
                "alias": "security-group",
                "description": "The security groups extension."
            },
            {
                "updated": "2013-02-07T10:00:00-00:00",
                "name": "L3 Agent Scheduler",
                "links": [],
                "alias": "l3_agent_scheduler",
                "description": "Schedule routers among l3 agents"
            },
            {
                "updated": "2013-02-07T10:00:00-00:00",
                "name": "Loadbalancer Agent Scheduler",
                "links": [],
                "alias": "lbaas_agent_scheduler",
                "description": "Schedule pools among lbaas agents"
            },
            {
                "updated": "2013-03-28T10:00:00-00:00",
                "name": "Neutron L3 Configurable external gateway mode",
                "links": [],
                "alias": "ext-gw-mode",
                "description":
                "Extension of the router abstraction for specifying whether"
                " SNAT should occur on the external gateway"
            },
            {
                "updated": "2014-02-03T10:00:00-00:00",
                "name": "Port Binding",
                "links": [],
                "alias": "binding",
                "description": "Expose port bindings of a virtual port to"
                " external application"
            },
            {
                "updated": "2012-09-07T10:00:00-00:00",
                "name": "Provider Network",
                "links": [],
                "alias": "provider",
                "description": "Expose mapping of virtual networks to"
                " physical networks"
            },
            {
                "updated": "2013-02-03T10:00:00-00:00",
                "name": "agent",
                "links": [],
                "alias": "agent",
                "description": "The agent management extension."
            },
            {
                "updated": "2012-07-29T10:00:00-00:00",
                "name": "Quota management support",
                "links": [],
                "alias": "quotas",
                "description": "Expose functions for quotas management per"
                " tenant"
            },
            {
                "updated": "2013-02-07T10:00:00-00:00",
                "name": "DHCP Agent Scheduler",
                "links": [],
                "alias": "dhcp_agent_scheduler",
                "description": "Schedule networks among dhcp agents"
            },
            {
                "updated": "2013-06-27T10:00:00-00:00",
                "name": "Multi Provider Network",
                "links": [],
                "alias": "multi-provider",
                "description": "Expose mapping of virtual networks to"
                " multiple physical networks"
            },
            {
                "updated": "2013-01-14T10:00:00-00:00",
                "name": "Neutron external network",
                "links": [],
                "alias": "external-net",
                "description": "Adds external network attribute to network"
                " resource."
            },
            {
                "updated": "2012-07-20T10:00:00-00:00",
                "name": "Neutron L3 Router",
                "links": [],
                "alias": "router",
                "description": "Router abstraction for basic L3 forwarding"
                " between L2 Neutron networks and access to external"
                " networks via a NAT gateway."
            },
            {
                "updated": "2013-07-23T10:00:00-00:00",
                "name": "Allowed Address Pairs",
                "links": [],
                "alias": "allowed-address-pairs",
                "description": "Provides allowed address pairs"
            },
            {
                "updated": "2013-03-17T12:00:00-00:00",
                "name": "Neutron Extra DHCP opts",
                "links": [],
                "alias": "extra_dhcp_opt",
                "description": "Extra options configuration for DHCP. For"
                " example PXE boot options to DHCP clients can be specified"
                " (e.g. tftp-server, server-ip-address, bootfile-name)"
            },
            {
                "updated": "2012-10-07T10:00:00-00:00",
                "name": "LoadBalancing service",
                "links": [],
                "alias": "lbaas",
                "description": "Extension for LoadBalancing service"
            },
            {
                "updated": "2013-02-01T10:00:00-00:00",
                "name": "Neutron Extra Route",
                "links": [],
                "alias": "extraroute",
                "description": "Extra routes configuration for L3 router"
            },
            {
                "updated": "2016-01-24T10:00:00-00:00",
                "name": "Neutron Port Data Plane Status",
                "links": [],
                "alias": "data-plane-status",
                "description": "Status of the underlying data plane."
            }
        ]
    }

    FAKE_EXTENSION_ALIAS = "service-type"

    def setUp(self):
        super(TestExtensionsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.extensions_client = extensions_client.ExtensionsClient(
            fake_auth, "network", "regionOne")

    def _test_list_extensions(self, bytes_body=False):
        self.check_service_client_function(
            self.extensions_client.list_extensions,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_EXTENSIONS,
            bytes_body,
            200)

    def _test_show_extension(self, bytes_body=False):
        self.check_service_client_function(
            self.extensions_client.show_extension,
            "tempest.lib.common.rest_client.RestClient.get",
            {"extension": self.FAKE_EXTENSIONS["extensions"][0]},
            bytes_body,
            200,
            ext_alias=self.FAKE_EXTENSION_ALIAS)

    def test_list_extensions_with_str_body(self):
        self._test_list_extensions()

    def test_list_extensions_with_bytes_body(self):
        self._test_list_extensions(bytes_body=True)

    def test_show_extension_with_str_body(self):
        self._test_show_extension()

    def test_show_extension_with_bytes_body(self):
        self._test_show_extension(bytes_body=True)



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from tempest.lib.api_schema.response.compute.v2_50 import quota_classes \
    as quota_classesv250

# Compute microversion 2.57:
# 1. injected_file_content_bytes, injected_file_path_bytes, injected_files
#    are removed from:
#      * GET /os-quota-class-sets/{id}
#      * PUT /os-quota-class-sets/{id}

get_quota_class_set = copy.deepcopy(quota_classesv250.get_quota_class_set)
update_quota_class_set = copy.deepcopy(
    quota_classesv250.update_quota_class_set)
for field in ['injected_file_content_bytes', 'injected_file_path_bytes',
              'injected_files']:
    get_quota_class_set['response_body']['properties']['quota_class_set'][
        'properties'].pop(field, None)
    get_quota_class_set['response_body']['properties']['quota_class_set'][
        'required'].remove(field)
    update_quota_class_set['response_body']['properties']['quota_class_set'][
        'properties'].pop(field, None)
    update_quota_class_set['response_body']['properties'][
        'quota_class_set']['required'].remove(field)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.identity.v3 import identity_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestIdentityClient(base.BaseServiceTest):
    FAKE_TOKEN = {
        "tokens": {
            "id": "cbc36478b0bd8e67e89",
            "name": "FakeToken",
            "type": "token",
        }
    }

    FAKE_API_INFO = {
        "name": "API_info",
        "type": "API",
        "description": "test_description"
    }

    FAKE_AUTH_PROJECTS = {
        "projects": [
            {
                "domain_id": "1789d1",
                "enabled": True,
                "id": "263fd9",
                "links": {
                    "self": "https://example.com/identity/v3/projects/263fd9"
                },
                "name": "Test Group"
            },
            {
                "domain_id": "1789d1",
                "enabled": True,
                "id": "50ef01",
                "links": {
                    "self": "https://example.com/identity/v3/projects/50ef01"
                },
                "name": "Build Group"
            }
        ],
        "links": {
            "self": "https://example.com/identity/v3/auth/projects",
            "previous": None,
            "next": None
        }
    }

    FAKE_AUTH_DOMAINS = {
        "domains": [
            {
                "description": "my domain description",
                "enabled": True,
                "id": "1789d1",
                "links": {
                    "self": "https://example.com/identity/v3/domains/1789d1"
                },
                "name": "my domain"
            },
            {
                "description": "description of my other domain",
                "enabled": True,
                "id": "43e8da",
                "links": {
                    "self": "https://example.com/identity/v3/domains/43e8da"
                },
                "name": "another domain"
            }
        ],
        "links": {
            "self": "https://example.com/identity/v3/auth/domains",
            "previous": None,
            "next": None
        }
    }

    def setUp(self):
        super(TestIdentityClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = identity_client.IdentityClient(fake_auth,
                                                     'identity',
                                                     'regionOne')

    def _test_show_api_description(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_api_description,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_API_INFO,
            bytes_body)

    def _test_show_token(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_token,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_TOKEN,
            bytes_body,
            resp_token="cbc36478b0bd8e67e89")

    def _test_list_auth_projects(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_auth_projects,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_AUTH_PROJECTS,
            bytes_body)

    def _test_list_auth_domains(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_auth_domains,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_AUTH_DOMAINS,
            bytes_body)

    def test_show_api_description_with_str_body(self):
        self._test_show_api_description()

    def test_show_api_description_with_bytes_body(self):
        self._test_show_api_description(bytes_body=True)

    def test_show_token_with_str_body(self):
        self._test_show_token()

    def test_show_token_with_bytes_body(self):
        self._test_show_token(bytes_body=True)

    def test_delete_token(self):
        self.check_service_client_function(
            self.client.delete_token,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            resp_token="cbc36478b0bd8e67e89",
            status=204)

    def test_check_token_existence(self):
        self.check_service_client_function(
            self.client.check_token_existence,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            resp_token="cbc36478b0bd8e67e89",
            status=200)

    def test_list_auth_projects_with_str_body(self):
        self._test_list_auth_projects()

    def test_list_auth_projects_with_bytes_body(self):
        self._test_list_auth_projects(bytes_body=True)

    def test_list_auth_domains_with_str_body(self):
        self._test_list_auth_domains()

    def test_list_auth_domains_with_bytes_body(self):
        self._test_list_auth_domains(bytes_body=True)



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import ports_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestPortsClient(base.BaseServiceTest):

    FAKE_CREATE_PORTS = {
        "port": {
            "binding:host_id": "4df8d9ff-6f6f-438f-90a1-ef660d4586ad",
            "binding:profile": {
                "local_link_information": [
                    {
                        "port_id": "Ethernet3/1",
                        "switch_id": "0a:1b:2c:3d:4e:5f",
                        "switch_info": "switch1"
                    }
                ]
            },
            "binding:vnic_type": "baremetal",
            "device_id": "d90a13da-be41-461f-9f99-1dbcf438fdf2",
            "device_owner": "baremetal:none",
            "dns_domain": "my-domain.org.",
            "dns_name": "myport",
            "qos_policy_id": "29d5e02e-d5ab-4929-bee4-4a9fc12e22ae",
            "uplink_status_propagation": False
        }
    }

    FAKE_PORTS = {
        "ports": [
            {
                "admin_state_up": True,
                "allowed_address_pairs": [],
                "created_at": "2016-03-08T20:19:41",
                "data_plane_status": None,
                "description": "",
                "device_id": "9ae135f4-b6e0-4dad-9e91-3c223e385824",
                "device_owner": "network:router_gateway",
                "dns_assignment": [
                    {
                        "hostname": "myport",
                        "ip_address": "172.24.4.2",
                        "fqdn": "myport.my-domain.org"
                    }
                ],
                "dns_domain": "my-domain.org.",
                "dns_name": "myport",
                "extra_dhcp_opts": [
                    {
                        "opt_value": "pxelinux.0",
                        "ip_version": 4,
                        "opt_name": "bootfile-name"
                    }
                ],
                "fixed_ips": [
                    {
                        "ip_address": "172.24.4.2",
                        "subnet_id":
                            "008ba151-0b8c-4a67-98b5-0d2b87666062"
                    }
                ],
                "id": "d80b1a3b-4fc1-49f3-952e-1e2ab7081d8b",
                "ip_allocation": "immediate",
                "mac_address": "fa:16:3e:58:42:ed",
                "name": "",
                "network_id": "70c1db1f-b701-45bd-96e0-a313ee3430b3",
                "project_id": "",
                "revision_number": 1,
                "security_groups": [],
                "status": "ACTIVE",
                "tags": ["tag1,tag2"],
                "tenant_id": "d6700c0c9ffa4f1cb322cd4a1f3906fa",
                "updated_at": "2016-03-08T20:19:41",
                "qos_network_policy_id":
                    "174dd0c1-a4eb-49d4-a807-ae80246d82f4",
                "qos_policy_id": "29d5e02e-d5ab-4929-bee4-4a9fc12e22ae",
                "port_security_enabled": False,
                "uplink_status_propagation": False
            },
            {
                "admin_state_up": True,
                "allowed_address_pairs": [],
                "created_at": "2016-03-08T20:19:41",
                "data_plane_status": None,
                "description": "",
                "device_id": "9ae135f4-b6e0-4dad-9e91-3c223e385824",
                "device_owner": "network:router_interface",
                "dns_assignment": [
                    {
                        "hostname": "myport2",
                        "ip_address": "10.0.0.1",
                        "fqdn": "myport2.my-domain.org"
                    }
                ],
                "dns_domain": "my-domain.org.",
                "dns_name": "myport2",
                "extra_dhcp_opts": [
                    {
                        "opt_value": "pxelinux.0",
                        "ip_version": 4,
                        "opt_name": "bootfile-name"
                    }
                ],
                "fixed_ips": [
                    {
                        "ip_address": "10.0.0.1",
                        "subnet_id":
                            "288bf4a1-51ba-43b6-9d0a-520e9005db17"
                    }
                ],
                "id": "f71a6703-d6de-4be1-a91a-a570ede1d159",
                "ip_allocation": "immediate",
                "mac_address": "fa:16:3e:bb:3c:e4",
                "name": "",
                "network_id": "f27aa545-cbdd-4907-b0c6-c9e8b039dcc2",
                "project_id": "d397de8a63f341818f198abb0966f6f3",
                "revision_number": 1,
                "security_groups": [],
                "status": "ACTIVE",
                "tags": ["tag1,tag2"],
                "tenant_id": "d397de8a63f341818f198abb0966f6f3",
                "updated_at": "2016-03-08T20:19:41",
                "qos_network_policy_id": None,
                "qos_policy_id": None,
                "port_security_enabled": False,
                "uplink_status_propagation": False
            }
        ]
    }

    FAKE_PORT_ID = "d80b1a3b-4fc1-49f3-952e-1e2ab7081d8b"

    FAKE_PORT1 = {
        "admin_state_up": True,
        "name": "",
        "network_id": "70c1db1f-b701-45bd-96e0-a313ee3430b3"
    }

    FAKE_PORT2 = {
        "admin_state_up": True,
        "name": "",
        "network_id": "f27aa545-cbdd-4907-b0c6-c9e8b039dcc2"
    }

    FAKE_PORTS_REQ = {
        "ports": [
            FAKE_PORT1,
            FAKE_PORT2
        ]
    }

    def setUp(self):
        super(TestPortsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.ports_client = ports_client.PortsClient(
            fake_auth, "network", "regionOne")

    def _test_list_ports(self, bytes_body=False):
        self.check_service_client_function(
            self.ports_client.list_ports,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_PORTS,
            bytes_body,
            200)

    def _test_create_port(self, bytes_body=False):
        self.check_service_client_function(
            self.ports_client.create_port,
            "tempest.lib.common.rest_client.RestClient.post",
            self.FAKE_CREATE_PORTS,
            bytes_body,
            201,
            **self.FAKE_PORT1)

    def _test_create_bulk_ports(self, bytes_body=False):
        self.check_service_client_function(
            self.ports_client.create_bulk_ports,
            "tempest.lib.common.rest_client.RestClient.post",
            self.FAKE_PORTS,
            bytes_body,
            201,
            ports=self.FAKE_PORTS_REQ)

    def _test_show_port(self, bytes_body=False):
        self.check_service_client_function(
            self.ports_client.show_port,
            "tempest.lib.common.rest_client.RestClient.get",
            {"port": self.FAKE_PORTS["ports"][0]},
            bytes_body,
            200,
            port_id=self.FAKE_PORT_ID)

    def _test_update_port(self, bytes_body=False):
        update_kwargs = {
            "admin_state_up": True,
            "device_id": "d90a13da-be41-461f-9f99-1dbcf438fdf2",
            "device_owner": "compute:nova",
            "name": "test-for-port-update"
        }

        resp_body = {
            "port": copy.deepcopy(
                self.FAKE_PORTS["ports"][0]
            )
        }
        resp_body["port"].update(update_kwargs)

        self.check_service_client_function(
            self.ports_client.update_port,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            port_id=self.FAKE_PORT_ID,
            **update_kwargs)

    def test_delete_port(self):
        self.check_service_client_function(
            self.ports_client.delete_port,
            "tempest.lib.common.rest_client.RestClient.delete",
            {},
            status=204,
            port_id=self.FAKE_PORT_ID)

    def test_list_ports_with_str_body(self):
        self._test_list_ports()

    def test_list_ports_with_bytes_body(self):
        self._test_list_ports(bytes_body=True)

    def test_create_port_with_str_body(self):
        self._test_create_port()

    def test_create_port_with_bytes_body(self):
        self._test_create_port(bytes_body=True)

    def test_create_bulk_port_with_str_body(self):
        self._test_create_bulk_ports()

    def test_create_bulk_port_with_bytes_body(self):
        self._test_create_bulk_ports(bytes_body=True)

    def test_show_port_with_str_body(self):
        self._test_show_port()

    def test_show_port_with_bytes_body(self):
        self._test_show_port(bytes_body=True)

    def test_update_port_with_str_body(self):
        self._test_update_port()

    def test_update_port_with_bytes_body(self):
        self._test_update_port(bytes_body=True)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


CONF = config.CONF


class VolumesAssistedSnapshotsTest(base.BaseV2ComputeAdminTest):
    """Test volume assisted snapshots"""

    create_default_network = True

    # TODO(gmann): Remove the admin access to service user
    # once nova change the default of this API to service
    # role. To merge the nova changing the policy default
    # we need to use token with admin as well as service
    # role and later we can use only service token.
    credentials = ['primary', 'admin', ['service_user', 'admin', 'service']]

    @classmethod
    def skip_checks(cls):
        super(VolumesAssistedSnapshotsTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(VolumesAssistedSnapshotsTest, cls).setup_clients()
        cls.assisted_v_client = (
            cls.os_service_user.assisted_volume_snapshots_client)
        cls.volumes_client = cls.os_admin.volumes_client_latest
        cls.servers_client = cls.os_admin.servers_client

    @decorators.idempotent_id('8aee84a3-1b1f-42e4-9b00-613931ccac24')
    def test_volume_assisted_snapshot_create_delete(self):
        """Test create/delete volume assisted snapshot"""
        volume = self.create_volume()
        self.addCleanup(self.delete_volume, volume['id'])
        validation_resources = self.get_class_validation_resources(
            self.os_primary)
        server = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until='SSHABLE'
        )
        # Attach created volume to server
        self.attach_volume(server, volume)
        snapshot_id = data_utils.rand_uuid()
        snapshot = self.assisted_v_client.create_assisted_volume_snapshot(
            volume_id=volume['id'], snapshot_id=snapshot_id,
            type='qcow2', new_file='new_file')['snapshot']
        self.assisted_v_client.delete_assisted_volume_snapshot(
            volume_id=volume['id'], snapshot_id=snapshot['id'])



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_serialization import jsonutils as json

from tempest.lib.services.volume.v3 import snapshot_manage_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSnapshotManageClient(base.BaseServiceTest):

    SNAPSHOT_MANAGE_REQUEST = {
        "snapshot": {
            "description": "snapshot-manage-description",
            "metadata": None,
            "ref": {
                "source-name": "_snapshot-22b71da0-94f9-4aca-ad45-7522b3fa96bb"
            },
            "name": "snapshot-managed",
            "volume_id": "7c064b34-1e4b-40bd-93ca-4ac5a973661b"
        }
    }

    SNAPSHOT_MANAGE_RESPONSE = {
        "snapshot": {
            "status": "creating",
            "description": "snapshot-manage-description",
            "updated_at": None,
            "volume_id": "32bafcc8-7109-42cd-8342-70d8de2bedef",
            "id": "8fd6eb9d-0a82-456d-b1ec-dea4ac7f1ee2",
            "size": 1,
            "name": "snapshot-managed",
            "created_at": "2017-07-11T10:07:58.000000",
            "metadata": {}
        }
    }

    def setUp(self):
        super(TestSnapshotManageClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = snapshot_manage_client.SnapshotManageClient(fake_auth,
                                                                  'volume',
                                                                  'regionOne')

    def _test_manage_snapshot(self, bytes_body=False):
        payload = json.dumps(self.SNAPSHOT_MANAGE_REQUEST, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(snapshot_manage_client.json,
                               'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.manage_snapshot,
                'tempest.lib.common.rest_client.RestClient.post',
                self.SNAPSHOT_MANAGE_RESPONSE,
                to_utf=bytes_body,
                status=202,
                mock_args=['os-snapshot-manage', payload],
                **self.SNAPSHOT_MANAGE_REQUEST['snapshot'])

    def test_manage_snapshot_with_str_body(self):
        self._test_manage_snapshot()

    def test_manage_snapshot_with_bytes_body(self):
        self._test_manage_snapshot(bytes_body=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import trunks_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestTrunksClient(base.BaseServiceTest):

    FAKE_TRUNK_ID = "dfbc2103-93cf-4edf-952a-ef6deb32ddc6"
    FAKE_PORT_ID = "1f04eb36-6c84-11eb-b0ab-4fc62961629d"
    FAKE_TRUNKS = {
        "trunks": [
            {
                "admin_state_up": True,
                "description": "",
                "id": "dfbc2103-93cf-4edf-952a-ef6deb32ddc6",
                "name": "trunk0",
                "port_id": "00130aab-bb51-42a1-a7c4-6703a3a43aa5",
                "project_id": "",
                "revision_number": 2,
                "status": "DOWN",
                "sub_ports": [
                    {
                        "port_id": "87d2483d-e5e6-483d-b5f0-81b9ed1d1a91",
                        "segmentation_id": 101,
                        "segmentation_type": "vlan"
                        }
                    ],
                "tags": [],
            },
            {
                "admin_state_up": True,
                "description": "",
                "id": "9eb0e72e-11d3-4295-bcaf-6c89008d9f0a",
                "name": "trunk1",
                "port_id": "035a12bf-2ae3-42ae-8ad6-9f70640cddde",
                "project_id": "",
                "revision_number": 2,
                "status": "DOWN",
                "sub_ports": [
                    {
                        "port_id": "cba839d5-02e2-4e09-b964-81356da78165",
                        "segmentation_id": 102,
                        "segmentation_type": "vlan"
                        }
                    ],
                "tags": [],
            },
        ]
    }

    FAKE_TRUNK_1 = {
        "name": "trunk0",
        "port_id": "00130aab-bb51-42a1-a7c4-6703a3a43aa5"
    }

    def setUp(self):
        super(TestTrunksClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.trunks_client = trunks_client.TrunksClient(
            fake_auth, "network", "regionOne")

    def _test_create_trunk(self, bytes_body=False):
        self.check_service_client_function(
            self.trunks_client.create_trunk,
            "tempest.lib.common.rest_client.RestClient.post",
            {"trunk": self.FAKE_TRUNKS["trunks"][0]},
            bytes_body,
            201,
            **self.FAKE_TRUNK_1)

    def _test_list_trunks(self, bytes_body=False):
        self.check_service_client_function(
            self.trunks_client.list_trunks,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_TRUNKS,
            bytes_body,
            200)

    def _test_show_trunk(self, bytes_body=False):
        self.check_service_client_function(
            self.trunks_client.show_trunk,
            "tempest.lib.common.rest_client.RestClient.get",
            {"trunk": self.FAKE_TRUNKS["trunks"][0]},
            bytes_body,
            200,
            trunk_id=self.FAKE_TRUNK_ID)

    def _test_update_trunk(self, bytes_body=False):
        update_kwargs = {
            "admin_state_up": True,
            "name": "new_trunk"
        }

        resp_body = {
            "trunk": copy.deepcopy(
                self.FAKE_TRUNKS["trunks"][0]
            )
        }
        resp_body["trunk"].update(update_kwargs)

        self.check_service_client_function(
            self.trunks_client.update_trunk,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            trunk_id=self.FAKE_TRUNK_ID,
            **update_kwargs)

    def _test_add_subports_to_trunk(self, bytes_body=False):
        sub_ports = [{
            "port_id": "f04eb36-6c84-11eb-b0ab-4fc62961629d",
            "segmentation_type": "vlan",
            "segmentation_id": "1001"
        }]
        resp_body = copy.deepcopy(self.FAKE_TRUNKS["trunks"][0])

        resp_body["sub_ports"].append(sub_ports)
        self.check_service_client_function(
            self.trunks_client.add_subports_to_trunk,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            trunk_id=self.FAKE_TRUNK_ID,
            sub_ports=sub_ports)

    def _test_delete_subports_from_trunk(self, bytes_body=False):
        fake_sub_ports = self.FAKE_TRUNKS['trunks'][0]['sub_ports']
        sub_ports = [
            {"port_id": fake_sub_ports[0]['port_id']}
        ]
        resp_body = copy.deepcopy(self.FAKE_TRUNKS["trunks"][0])

        resp_body['sub_ports'] = []
        self.check_service_client_function(
            self.trunks_client.delete_subports_from_trunk,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            trunk_id=self.FAKE_TRUNK_ID,
            sub_ports=sub_ports)

    def test_create_trunk_with_str_body(self):
        self._test_create_trunk()

    def test_create_trunk_with_bytes_body(self):
        self._test_create_trunk(bytes_body=True)

    def test_list_trunks_with_str_body(self):
        self._test_list_trunks()

    def test_list_trunks_with_bytes_body(self):
        self._test_list_trunks(bytes_body=True)

    def test_show_trunk_with_str_body(self):
        self._test_show_trunk()

    def test_show_trunk_with_bytes_body(self):
        self._test_show_trunk(bytes_body=True)

    def test_update_trunk_with_str_body(self):
        self._test_update_trunk()

    def test_update_trunk_with_bytes_body(self):
        self._test_update_trunk(bytes_body=True)

    def test_add_subports_to_trunk_str_body(self):
        self._test_add_subports_to_trunk()

    def test_add_subports_to_trunk_bytes_body(self):
        self._test_add_subports_to_trunk(bytes_body=True)

    def test_delete_subports_from_trunk_str_body(self):
        self._test_delete_subports_from_trunk()

    def test_delete_subports_from_trunk_bytes_body(self):
        self._test_delete_subports_from_trunk(bytes_body=True)

    def test_delete_trunk(self):
        self.check_service_client_function(
            self.trunks_client.delete_trunk,
            "tempest.lib.common.rest_client.RestClient.delete",
            {},
            status=204,
            trunk_id=self.FAKE_TRUNK_ID)



# Copyright 2016 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.lib.services.object_storage import capabilities_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestCapabilitiesClient(base.BaseServiceTest):

    def setUp(self):
        super(TestCapabilitiesClient, self).setUp()
        self.fake_auth = fake_auth_provider.FakeAuthProvider()
        self.url = self.fake_auth.base_url(None)
        self.client = capabilities_client.CapabilitiesClient(
            self.fake_auth, 'swift', 'region1')

    def _test_list_capabilities(self, bytes_body=False):
        resp = {
            "swift": {
                "version": "1.11.0"
            },
            "slo": {
                "max_manifest_segments": 1000,
                "max_manifest_size": 2097152,
                "min_segment_size": 1
            },
            "staticweb": {},
            "tempurl": {}
        }
        self.check_service_client_function(
            self.client.list_capabilities,
            'tempest.lib.common.rest_client.RestClient.raw_request',
            resp,
            bytes_body)

    def test_list_capabilities_with_str_body(self):
        self._test_list_capabilities()

    def test_list_capabilities_with_bytes_body(self):
        self._test_list_capabilities(True)



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import services


list_services = copy.deepcopy(services.list_services)
list_services['response_body']['properties']['services']['items'][
    'properties']['forced_down'] = {'type': 'boolean'}
list_services['response_body']['properties']['services']['items'][
    'required'].append('forced_down')

update_forced_down = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'service': {
                'type': 'object',
                'properties': {
                    'binary': {'type': 'string'},
                    'host': {'type': 'string'},
                    'forced_down': {'type': 'boolean'}
                },
                'additionalProperties': False,
                'required': ['binary', 'host', 'forced_down']
            }
        },
        'additionalProperties': False,
        'required': ['service']
    }
}

# **** Schemas unchanged in microversion 2.11 since microversion 2.1 ****
# Note(felipemonteiro): Below are the unchanged schema in this microversion. We
# need to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
enable_disable_service = copy.deepcopy(services.enable_disable_service)
disable_log_reason = copy.deepcopy(services.disable_log_reason)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest.lib import decorators


class DeleteServersAdminTestJSON(base.BaseV2ComputeAdminTest):
    """Test deletion of servers"""

    # NOTE: Server creations of each test class should be under 10
    # for preventing "Quota exceeded for instances".

    @classmethod
    def setup_clients(cls):
        super(DeleteServersAdminTestJSON, cls).setup_clients()
        cls.non_admin_client = cls.servers_client
        cls.admin_client = cls.os_admin.servers_client

    @decorators.idempotent_id('99774678-e072-49d1-9d2a-49a59bc56063')
    def test_delete_server_while_in_error_state(self):
        """Delete a server while it's VM state is error"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.admin_client.reset_state(server['id'], state='error')
        # Verify server's state
        server = self.non_admin_client.show_server(server['id'])['server']
        self.assertEqual(server['status'], 'ERROR')
        self.non_admin_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client,
                                            server['id'],
                                            ignore_error=True)

    @decorators.idempotent_id('73177903-6737-4f27-a60c-379e8ae8cf48')
    def test_admin_delete_servers_of_others(self):
        """Administrator can delete servers of others"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.admin_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client, server['id'])



# Copyright 2017 NTT Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import server_groups

# Compute microversion 2.13:
# 1. New attributes in 'server_group' dict.
#      'project_id', 'user_id'
common_server_group = copy.deepcopy(server_groups.common_server_group)
common_server_group['properties']['project_id'] = {'type': 'string'}
common_server_group['properties']['user_id'] = {'type': 'string'}
common_server_group['required'].append('project_id')
common_server_group['required'].append('user_id')

create_show_server_group = copy.deepcopy(
    server_groups.create_show_server_group)
create_show_server_group['response_body']['properties'][
    'server_group'] = common_server_group

delete_server_group = copy.deepcopy(server_groups.delete_server_group)

list_server_groups = copy.deepcopy(server_groups.list_server_groups)
list_server_groups['response_body']['properties']['server_groups'][
    'items'] = common_server_group



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class TestVolumeMigrateRetypeAttached(manager.ScenarioTest):

    """This test case attempts to reproduce the following steps:

     * Create 2 volume types representing 2 different backends
     * Create in Cinder some bootable volume importing a Glance image using
     *   volume_type_1
     * Boot an instance from the bootable volume
     * Write to the volume
     * Perform a cinder retype --on-demand of the volume to type of backend #2
     * Check written content of migrated volume
     * Check the type of the volume has been updated.
     * Check the volume is still in-use and the migration was successful.
     * Check that the same volume is attached to the instance.
    """

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(TestVolumeMigrateRetypeAttached, cls).setup_clients()
        cls.admin_volumes_client = cls.os_admin.volumes_client_latest

    @classmethod
    def skip_checks(cls):
        super(TestVolumeMigrateRetypeAttached, cls).skip_checks()
        if not CONF.service_available.cinder:
            raise cls.skipException("Cinder is not available")
        if not CONF.volume_feature_enabled.multi_backend:
            raise cls.skipException("Cinder multi-backend feature disabled")

        if len(set(CONF.volume.backend_names)) < 2:
            raise cls.skipException("Requires at least two different "
                                    "backend names")

    def _boot_instance_from_volume(self, vol_id, keypair, security_group):

        key_name = keypair['name']
        security_groups = [{'name': security_group['name']}]
        block_device_mapping = [{'device_name': 'vda', 'volume_id': vol_id,
                                 'delete_on_termination': False}]

        return self.create_server(image_id='',
                                  key_name=key_name,
                                  security_groups=security_groups,
                                  block_device_mapping=block_device_mapping)

    def _create_volume_types(self):
        backend_names = CONF.volume.backend_names

        backend_source = backend_names[0]
        backend_dest = backend_names[1]

        source_body = self.create_volume_type(backend_name=backend_source)
        dest_body = self.create_volume_type(backend_name=backend_dest)

        LOG.info("Created Volume types: %(src)s -> %(src_backend)s, %(dst)s "
                 "-> %(dst_backend)s", {'src': source_body['name'],
                                        'src_backend': backend_source,
                                        'dst': dest_body['name'],
                                        'dst_backend': backend_dest})
        return ({'name': source_body['name'], 'host': backend_source},
                {'name': dest_body['name'], 'host': backend_dest})

    def _volume_retype_with_migration(self, volume_id, new_volume_type):
        # NOTE: The 'on-demand' migration requires admin operation, so
        # admin_volumes_client() should be used here.
        migration_policy = 'on-demand'
        self.admin_volumes_client.retype_volume(
            volume_id, new_type=new_volume_type,
            migration_policy=migration_policy)
        waiters.wait_for_volume_retype(self.volumes_client,
                                       volume_id, new_volume_type)

    def _test_volume_retype_attached(self, dev_name=None):
        LOG.info("Creating keypair and security group")
        keypair = self.create_keypair()
        security_group = self.create_security_group()

        # create volume types
        LOG.info("Creating Volume types")
        source_type, dest_type = self._create_volume_types()

        if dev_name is None:
            # create an instance from volume
            LOG.info("Booting instance from volume")
            volume_id = self.create_volume(
                imageRef=CONF.compute.image_ref,
                volume_type=source_type['name'])['id']

            instance = self._boot_instance_from_volume(volume_id, keypair,
                                                       security_group)
        else:
            LOG.info("Booting instance from image and attaching data volume")
            key_name = keypair['name']
            security_groups = [{'name': security_group['name']}]
            instance = self.create_server(key_name=key_name,
                                          security_groups=security_groups)
            volume = self.create_volume(volume_type=source_type['name'])
            volume_id = volume['id']
            volume = self.nova_volume_attach(instance, volume)

        # write content to volume on instance
        LOG.info("Setting timestamp in instance %s", instance['id'])
        ip_instance = self.get_server_ip(instance)
        timestamp = self.create_timestamp(ip_instance,
                                          dev_name=dev_name,
                                          private_key=keypair['private_key'],
                                          server=instance)

        # retype volume with migration from backend #1 to backend #2
        LOG.info("Retyping Volume %s to new type %s", volume_id,
                 dest_type['name'])
        # This method calls for the retype of the volume before calling a
        # waiter that asserts that the volume type has changed successfully.
        self._volume_retype_with_migration(volume_id, dest_type['name'])

        # check the content of written file
        LOG.info("Getting timestamp in postmigrated instance %s",
                 instance['id'])
        timestamp2 = self.get_timestamp(ip_instance,
                                        dev_name=dev_name,
                                        private_key=keypair['private_key'],
                                        server=instance)
        self.assertEqual(timestamp, timestamp2)

        # Assert that the volume is on the new host, is still in-use and has a
        # migration_status of success
        volume = self.admin_volumes_client.show_volume(volume_id)['volume']
        # dest_type is host@backend, os-vol-host-attr:host is host@backend#type
        self.assertIn(dest_type['host'], volume['os-vol-host-attr:host'])
        self.assertEqual('in-use', volume['status'])
        self.assertEqual('success', volume['migration_status'])

        # Assert that the same volume id is attached to the instance, ensuring
        # the os-migrate_volume_completion Cinder API has been called.
        attached_volumes = self.servers_client.list_volume_attachments(
            instance['id'])['volumeAttachments']
        self.assertEqual(volume_id, attached_volumes[0]['id'])

        # Reboot the instance and verify it boots successfully
        LOG.info("Hard rebooting instance %s", instance['id'])
        self.servers_client.reboot_server(instance['id'], type='HARD')
        waiters.wait_for_server_status(
            self.servers_client, instance['id'], 'ACTIVE')

        # check the content of written file to verify the instance is working
        # after being rebooted
        LOG.info("Getting timestamp in postmigrated rebooted instance %s",
                 instance['id'])
        timestamp2 = self.get_timestamp(ip_instance,
                                        dev_name=dev_name,
                                        private_key=keypair['private_key'],
                                        server=instance)
        self.assertEqual(timestamp, timestamp2)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('deadd2c2-beef-4dce-98be-f86765ff311b')
    @utils.services('compute', 'volume')
    def test_volume_retype_attached(self):
        self._test_volume_retype_attached()

    @decorators.attr(type='slow')
    @decorators.idempotent_id('122e070c-a5b2-470c-af2b-81e9dbefb9e8')
    @utils.services('compute', 'volume')
    def test_volume_retype_attached_data_volume(self):
        self._test_volume_retype_attached(dev_name='vdb')

    def _test_volume_migrate_attached(self, dev_name=None):
        LOG.info("Creating keypair and security group")
        keypair = self.create_keypair()
        security_group = self.create_security_group()

        LOG.info("Creating volume")
        # Create a unique volume type to avoid using the backend default
        migratable_type = self.create_volume_type()['name']

        if dev_name is None:
            volume_id = self.create_volume(imageRef=CONF.compute.image_ref,
                                           volume_type=migratable_type)['id']
            LOG.info("Booting instance from volume")
            instance = self._boot_instance_from_volume(volume_id, keypair,
                                                       security_group)
        else:
            LOG.info("Booting instance from image and attaching data volume")
            key_name = keypair['name']
            security_groups = [{'name': security_group['name']}]
            instance = self.create_server(key_name=key_name,
                                          security_groups=security_groups)
            volume = self.create_volume(volume_type=migratable_type)
            volume_id = volume['id']
            volume = self.nova_volume_attach(instance, volume)

        # Identify the source and destination hosts for the migration
        volume = self.admin_volumes_client.show_volume(volume_id)['volume']
        src_host = volume['os-vol-host-attr:host']

        # Select the first c-vol host that isn't hosting the volume as the dest
        # host['host_name'] should take the format of host@backend.
        # src_host should take the format of host@backend#type
        hosts = self.admin_volumes_client.list_hosts()['hosts']
        for host in hosts:
            if (host['service'] == 'cinder-volume' and
                not src_host.startswith(host['host_name'])):
                dest_host = host['host_name']
                break

        ip_instance = self.get_server_ip(instance)
        timestamp = self.create_timestamp(ip_instance,
                                          dev_name=dev_name,
                                          private_key=keypair['private_key'],
                                          server=instance)

        LOG.info("Migrating Volume %s from host %s to host %s",
                 volume_id, src_host, dest_host)
        self.admin_volumes_client.migrate_volume(volume_id, host=dest_host)

        # This waiter asserts that the migration_status is success and that
        # the volume has moved to the dest_host
        waiters.wait_for_volume_migration(self.admin_volumes_client, volume_id,
                                          dest_host)

        # check the content of written file
        LOG.info("Getting timestamp in postmigrated instance %s",
                 instance['id'])
        timestamp2 = self.get_timestamp(ip_instance,
                                        dev_name=dev_name,
                                        private_key=keypair['private_key'],
                                        server=instance)
        self.assertEqual(timestamp, timestamp2)

        # Assert that the volume is in-use
        volume = self.admin_volumes_client.show_volume(volume_id)['volume']
        self.assertEqual('in-use', volume['status'])

        # Assert that the same volume id is attached to the instance, ensuring
        # the os-migrate_volume_completion Cinder API has been called
        attached_volumes = self.servers_client.list_volume_attachments(
            instance['id'])['volumeAttachments']
        attached_volume_id = attached_volumes[0]['id']
        self.assertEqual(volume_id, attached_volume_id)

        # Reboot the instance and verify it boots successfully
        LOG.info("Hard rebooting instance %s", instance['id'])
        self.servers_client.reboot_server(instance['id'], type='HARD')
        waiters.wait_for_server_status(
            self.servers_client, instance['id'], 'ACTIVE')

        # check the content of written file to verify the instance is working
        # after being rebooted
        LOG.info("Getting timestamp in postmigrated rebooted instance %s",
                 instance['id'])
        timestamp2 = self.get_timestamp(ip_instance,
                                        dev_name=dev_name,
                                        private_key=keypair['private_key'],
                                        server=instance)
        self.assertEqual(timestamp, timestamp2)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('fe47b1ed-640e-4e3b-a090-200e25607362')
    @utils.services('compute', 'volume')
    def test_volume_migrate_attached(self):
        self._test_volume_migrate_attached()

    @decorators.attr(type='slow')
    @decorators.idempotent_id('1b8661cb-db93-4110-860b-201295027b78')
    @utils.services('compute', 'volume')
    def test_volume_migrate_attached_data_volume(self):
        self._test_volume_migrate_attached(dev_name='vdb')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types
from tempest.lib.api_schema.response.compute.v2_57 import servers as servers257

# microversion 2.58 added updated_at to the response
show_instance_action = copy.deepcopy(servers257.show_instance_action)
show_instance_action['response_body']['properties']['instanceAction'][
    'properties']['updated_at'] = parameter_types.date_time
show_instance_action['response_body']['properties']['instanceAction'][
    'required'].append('updated_at')

# Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
list_servers = copy.deepcopy(servers257.list_servers)
show_server_diagnostics = copy.deepcopy(servers257.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers257.get_remote_consoles)
list_tags = copy.deepcopy(servers257.list_tags)
update_all_tags = copy.deepcopy(servers257.update_all_tags)
delete_all_tags = copy.deepcopy(servers257.delete_all_tags)
check_tag_existence = copy.deepcopy(servers257.check_tag_existence)
update_tag = copy.deepcopy(servers257.update_tag)
delete_tag = copy.deepcopy(servers257.delete_tag)
get_server = copy.deepcopy(servers257.get_server)
list_servers_detail = copy.deepcopy(servers257.list_servers_detail)
update_server = copy.deepcopy(servers257.update_server)
rebuild_server = copy.deepcopy(servers257.rebuild_server)
rebuild_server_with_admin_pass = copy.deepcopy(
    servers257.rebuild_server_with_admin_pass)
attach_volume = copy.deepcopy(servers257.attach_volume)
show_volume_attachment = copy.deepcopy(servers257.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers257.list_volume_attachments)
create_backup = copy.deepcopy(servers257.create_backup)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class VolumesActionsTest(base.BaseVolumeTest):
    """Test volume actions"""

    create_default_network = True

    @classmethod
    def resource_setup(cls):
        super(VolumesActionsTest, cls).resource_setup()

        # Create a test shared volume for attach/detach tests
        cls.volume = cls.create_volume()

    @decorators.idempotent_id('fff42874-7db5-4487-a8e1-ddda5fb5288d')
    @decorators.attr(type='smoke')
    @utils.services('compute')
    def test_attach_detach_volume_to_instance(self):
        """Test attaching and detaching volume to instance"""
        # Create a server
        server = self.create_server()
        # Volume is attached and detached successfully from an instance
        self.volumes_client.attach_volume(self.volume['id'],
                                          instance_uuid=server['id'],
                                          mountpoint='/dev/%s' %
                                          CONF.compute.volume_device_name)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'in-use')
        self.volumes_client.detach_volume(self.volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')

    @decorators.idempotent_id('63e21b4c-0a0c-41f6-bfc3-7c2816815599')
    def test_volume_bootable(self):
        """Test setting and retrieving bootable flag of a volume"""
        for bool_bootable in [True, False]:
            self.volumes_client.set_bootable_volume(self.volume['id'],
                                                    bootable=bool_bootable)
            fetched_volume = self.volumes_client.show_volume(
                self.volume['id'])['volume']
            # Get Volume information
            # NOTE(masayukig): 'bootable' is "true" or "false" in the current
            # cinder implementation. So we need to cast boolean values to str
            # and make it lower to compare here.
            self.assertEqual(str(bool_bootable).lower(),
                             fetched_volume['bootable'])

    @decorators.idempotent_id('9516a2c8-9135-488c-8dd6-5677a7e5f371')
    @utils.services('compute')
    def test_get_volume_attachment(self):
        """Test getting volume attachments

        Attach a volume to a server, and then retrieve volume's attachments
        info.
        """
        # Create a server
        server = self.create_server()
        # Verify that a volume's attachment information is retrieved
        self.volumes_client.attach_volume(self.volume['id'],
                                          instance_uuid=server['id'],
                                          mountpoint='/dev/%s' %
                                          CONF.compute.volume_device_name)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'],
                                                'in-use')
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.volumes_client,
                        self.volume['id'], 'available')
        self.addCleanup(self.volumes_client.detach_volume, self.volume['id'])
        volume = self.volumes_client.show_volume(self.volume['id'])['volume']
        attachment = volume['attachments'][0]

        self.assertEqual('/dev/%s' %
                         CONF.compute.volume_device_name,
                         attachment['device'])
        self.assertEqual(server['id'], attachment['server_id'])
        self.assertEqual(self.volume['id'], attachment['id'])
        self.assertEqual(self.volume['id'], attachment['volume_id'])

    @decorators.idempotent_id('d8f1ca95-3d5b-44a3-b8ca-909691c9532d')
    @utils.services('image')
    def test_volume_upload(self):
        """Test uploading volume to create an image"""
        # NOTE(gfidente): the volume uploaded in Glance comes from setUpClass,
        # it is shared with the other tests. After it is uploaded in Glance,
        # there is no way to delete it from Cinder, so we delete it from Glance
        # using the Glance images_client and from Cinder via tearDownClass.
        image_name = data_utils.rand_name(self.__class__.__name__ + '-Image',
                                          prefix=CONF.resource_name_prefix)
        body = self.volumes_client.upload_volume(
            self.volume['id'], image_name=image_name,
            disk_format=CONF.volume.disk_format)['os-volume_upload_image']
        image_id = body["image_id"]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.images_client.delete_image,
                        image_id)
        waiters.wait_for_image_status(self.images_client, image_id, 'active')
        # This is required for the optimized upload volume path.
        # New location APIs are async so we need to wait for the location
        # import task to complete.
        # This should work with old location API since we don't fail if there
        # are no tasks for the image
        waiters.wait_for_image_tasks_status(self.images_client,
                                            image_id, 'success')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')

        image_info = self.images_client.show_image(image_id)
        self.assertEqual(image_name, image_info['name'])
        self.assertEqual(CONF.volume.disk_format, image_info['disk_format'])

    @decorators.idempotent_id('92c4ef64-51b2-40c0-9f7e-4749fbaaba33')
    def test_reserve_unreserve_volume(self):
        """Test reserving and unreserving volume"""
        # Mark volume as reserved.
        self.volumes_client.reserve_volume(self.volume['id'])
        # To get the volume info
        body = self.volumes_client.show_volume(self.volume['id'])['volume']
        self.assertIn('attaching', body['status'])
        # Unmark volume as reserved.
        self.volumes_client.unreserve_volume(self.volume['id'])
        # To get the volume info
        body = self.volumes_client.show_volume(self.volume['id'])['volume']
        self.assertIn('available', body['status'])

    @decorators.idempotent_id('fff74e1e-5bd3-4b33-9ea9-24c103bc3f59')
    def test_volume_readonly_update(self):
        """Test updating and retrieve volume's readonly flag"""
        for readonly in [True, False]:
            # Update volume readonly
            self.volumes_client.update_volume_readonly(self.volume['id'],
                                                       readonly=readonly)
            # Get Volume information
            fetched_volume = self.volumes_client.show_volume(
                self.volume['id'])['volume']
            # NOTE(masayukig): 'readonly' is "True" or "False" in the current
            # cinder implementation. So we need to cast boolean values to str
            # to compare here.
            self.assertEqual(str(readonly),
                             fetched_volume['metadata']['readonly'])



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute.keypairs import test_keypairs
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class KeyPairsV22TestJSON(test_keypairs.KeyPairsV2TestJSON):
    """Test keypairs API with compute microversion greater than 2.1"""

    min_microversion = '2.2'
    max_microversion = 'latest'

    def _check_keypair_type(self, keypair, keypair_type):
        if keypair_type is None:
            keypair_type = 'ssh'
        self.assertEqual(keypair_type, keypair['type'])

    def _test_keypairs_create_list_show(self, keypair_type=None):
        k_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='keypair')
        keypair = self.create_keypair(k_name, keypair_type=keypair_type)
        # Verify whether 'type' is present in keypair create response of
        # version 2.2 and it is with default value 'ssh'.
        self._check_keypair_type(keypair, keypair_type)
        keypair_detail = self.keypairs_client.show_keypair(k_name)['keypair']
        self._check_keypair_type(keypair_detail, keypair_type)
        fetched_list = self.keypairs_client.list_keypairs()['keypairs']
        for keypair in fetched_list:
            # Verify whether 'type' is present in keypair list response of
            # version 2.2 and it is with default value 'ssh'.
            if keypair['keypair']['name'] == k_name:
                self._check_keypair_type(keypair['keypair'], keypair_type)

    @decorators.idempotent_id('8726fa85-7f98-4b20-af9e-f710a4f3391c')
    def test_keypairsv22_create_list_show(self):
        """Test create/list/show keypair"""
        self._test_keypairs_create_list_show()

    @decorators.idempotent_id('89d59d43-f735-441a-abcf-0601727f47b6')
    def test_keypairsv22_create_list_show_with_type(self):
        """Test create/list/show keypair with keypair type"""
        keypair_type = 'x509'
        self._test_keypairs_create_list_show(keypair_type=keypair_type)



# Copyright 2013 IBM Corp.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import os
import re

from hacking import core


PYTHON_CLIENTS = ['cinder', 'glance', 'keystone', 'nova', 'swift', 'neutron',
                  'ironic', 'heat', 'sahara']

PYTHON_CLIENT_RE = re.compile('import (%s)client' % '|'.join(PYTHON_CLIENTS))
TEST_DEFINITION = re.compile(r'^\s*def test.*')
SETUP_TEARDOWN_CLASS_DEFINITION = re.compile(r'^\s+def (setUp|tearDown)Class')
SCENARIO_DECORATOR = re.compile(r'\s*@.*services\((.*)\)')
RAND_NAME_HYPHEN_RE = re.compile(r".*rand_name\(.+[\-\_][\"\']\)")
mutable_default_args = re.compile(r"^\s*def .+\((.+=\{\}|.+=\[\])")
TESTTOOLS_SKIP_DECORATOR = re.compile(r'\s*@testtools\.skip\((.*)\)')
METHOD = re.compile(r"^    def .+")
METHOD_GET_RESOURCE = re.compile(r"^\s*def (list|show)\_.+")
METHOD_DELETE_RESOURCE = re.compile(r"^\s*def delete_.+")
CLASS = re.compile(r"^class .+")
EX_ATTRIBUTE = re.compile(r'(\s+|\()(e|ex|exc|exception).message(\s+|\))')
NEGATIVE_TEST_DECORATOR = re.compile(
    r'\s*@decorators\.attr\(type=.*negative.*\)')
_HAVE_NEGATIVE_DECORATOR = False


@core.flake8ext
def import_no_clients_in_api_and_scenario_tests(logical_line, filename):
    """Check for client imports from tempest/api & tempest/scenario tests

    T102: Cannot import OpenStack python clients
    """

    if "tempest/api" in filename or "tempest/scenario" in filename:
        res = PYTHON_CLIENT_RE.match(logical_line)
        if res:
            return (logical_line.find(res.group(1)),
                    ("T102: python clients import not allowed"
                     " in tempest/api/* or tempest/scenario/* tests"))


@core.flake8ext
def scenario_tests_need_service_tags(logical_line, filename,
                                     previous_logical):
    """Check that scenario tests have service tags

    T104: Scenario tests require a services decorator
    """

    if 'tempest/scenario/' in filename and '/test_' in filename:
        if TEST_DEFINITION.match(logical_line):
            if not SCENARIO_DECORATOR.match(previous_logical):
                return (logical_line.find('def'),
                        "T104: Scenario tests require a service decorator")


@core.flake8ext
def no_setup_teardown_class_for_tests(logical_line, filename, noqa):

    if noqa:
        return

    if 'tempest/test.py' in filename or 'tempest/lib/' in filename:
        return

    if SETUP_TEARDOWN_CLASS_DEFINITION.match(logical_line):
        return (logical_line.find('def'),
                "T105: (setUp|tearDown)Class can not be used in tests")


@core.flake8ext
def service_tags_not_in_module_path(logical_line, filename):
    """Check that a service tag isn't in the module path

    A service tag should only be added if the service name isn't already in
    the module path.

    T107
    """
    # NOTE(mtreinish) Scenario tests always need service tags, but subdirs are
    # created for services like heat which would cause false negatives for
    # those tests, so just exclude the scenario tests.
    if 'tempest/scenario' not in filename:
        matches = SCENARIO_DECORATOR.match(logical_line)
        if matches:
            services = matches.group(1).split(',')
            for service in services:
                service_name = service.strip().strip("'")
                modulepath = os.path.split(filename)[0]
                if service_name in modulepath:
                    return (logical_line.find(service_name),
                            "T107: service tag should not be in path")


@core.flake8ext
def no_hyphen_at_end_of_rand_name(logical_line, filename):
    """Check no hyphen at the end of rand_name() argument

    T108
    """
    msg = "T108: hyphen should not be specified at the end of rand_name()"
    if RAND_NAME_HYPHEN_RE.match(logical_line):
        return 0, msg


@core.flake8ext
def no_mutable_default_args(logical_line):
    """Check that mutable object isn't used as default argument

    N322: Method's default argument shouldn't be mutable
    """
    msg = "N322: Method's default argument shouldn't be mutable!"
    if mutable_default_args.match(logical_line):
        yield (0, msg)


@core.flake8ext
def no_testtools_skip_decorator(logical_line):
    """Check that methods do not have the testtools.skip decorator

    T109
    """
    if TESTTOOLS_SKIP_DECORATOR.match(logical_line):
        yield (0, "T109: Cannot use testtools.skip decorator; instead use "
               "decorators.skip_because from tempest.lib")


def _common_service_clients_check(logical_line, filename, noqa):
    if noqa:
        return False

    if not re.match('tempest/(lib/)?services/.*', filename):
        return False

    if not METHOD.match(logical_line):
        return False

    return True


@core.flake8ext
def get_resources_on_service_clients(logical_line, filename,
                                     line_number, lines, noqa):
    """Check that service client names of GET should be consistent

    T110
    """
    if not _common_service_clients_check(logical_line, filename, noqa):
        return

    for line in lines[line_number:]:
        if METHOD.match(line) or CLASS.match(line):
            # the end of a method
            return

        if 'self.get(' not in line and ('self.show_resource(' not in line and
                                        'self.list_resources(' not in line):
            continue

        if METHOD_GET_RESOURCE.match(logical_line):
            return

        msg = ("T110: [GET /resources] methods should be list_<resource name>s"
               " or show_<resource name>")
        yield (0, msg)


@core.flake8ext
def delete_resources_on_service_clients(logical_line, filename,
                                        line_number, lines, noqa):
    """Check that service client names of DELETE should be consistent

    T111
    """
    if not _common_service_clients_check(logical_line, filename, noqa):
        return

    for line in lines[line_number:]:
        if METHOD.match(line) or CLASS.match(line):
            # the end of a method
            return

        if 'self.delete(' not in line and 'self.delete_resource(' not in line:
            continue

        if METHOD_DELETE_RESOURCE.match(logical_line):
            return

        msg = ("T111: [DELETE /resources/<id>] methods should be "
               "delete_<resource name>")
        yield (0, msg)


@core.flake8ext
def dont_import_local_tempest_into_lib(logical_line, filename):
    """Check that tempest.lib should not import local tempest code

    T112
    """
    if 'tempest/lib/' not in filename:
        return

    if not ('from tempest' in logical_line or
            'import tempest' in logical_line):
        return

    if ('from tempest.lib' in logical_line or
            'import tempest.lib' in logical_line):
        return

    msg = ("T112: tempest.lib should not import local tempest code to avoid "
           "circular dependency")
    yield (0, msg)


@core.flake8ext
def use_rand_uuid_instead_of_uuid4(logical_line, filename):
    """Check that tests use data_utils.rand_uuid() instead of uuid.uuid4()

    T113
    """
    if 'tempest/lib/' in filename:
        return

    if 'uuid.uuid4()' not in logical_line:
        return

    msg = ("T113: Tests should use data_utils.rand_uuid()/rand_uuid_hex() "
           "instead of uuid.uuid4()/uuid.uuid4().hex")
    yield (0, msg)


@core.flake8ext
def dont_use_config_in_tempest_lib(logical_line, filename):
    """Check that tempest.lib doesn't use tempest config

    T114
    """

    if 'tempest/lib/' not in filename:
        return

    if ('tempest.config' in logical_line or
            'from tempest import config' in logical_line or
            'oslo_config' in logical_line):
        msg = ('T114: tempest.lib can not have any dependency on tempest '
               'config.')
        yield (0, msg)


@core.flake8ext
def dont_put_admin_tests_on_nonadmin_path(logical_line,
                                          filename):
    """Check admin tests should exist under admin path

    T115
    """

    if 'tempest/api/' not in filename:
        return

    if not re.match(r'class .*Test.*\(.*Admin.*\):', logical_line):
        return

    if not re.match(r'.\/tempest\/api\/.*\/admin\/.*', filename):
        msg = 'T115: All admin tests should exist under admin path.'
        yield (0, msg)


@core.flake8ext
def unsupported_exception_attribute_PY3(logical_line):
    """Check Unsupported 'message' exception attribute in PY3

    T116
    """
    result = EX_ATTRIBUTE.search(logical_line)
    msg = ("[T116] Unsupported 'message' Exception attribute in PY3")
    if result:
        yield (0, msg)


@core.flake8ext
def negative_test_attribute_always_applied_to_negative_tests(logical_line,
                                                             filename):
    """Check ``@decorators.attr(type=['negative'])`` applied to negative tests.

    T117
    """
    global _HAVE_NEGATIVE_DECORATOR

    if re.match(r'.\/tempest\/api\/.*_negative.*', filename):

        if NEGATIVE_TEST_DECORATOR.match(logical_line):
            _HAVE_NEGATIVE_DECORATOR = True
            return

        if TEST_DEFINITION.match(logical_line):
            if not _HAVE_NEGATIVE_DECORATOR:
                yield (
                    0, "T117: Must apply `@decorators.attr(type=['negative'])`"
                       " to all negative API tests"
                )
            _HAVE_NEGATIVE_DECORATOR = False


@core.flake8ext
def no_log_warn(logical_line):
    """Disallow 'LOG.warn('

    Use LOG.warning() instead of Deprecated LOG.warn().
    https://docs.python.org/3/library/logging.html#logging.warning
    """

    msg = ("T118: LOG.warn is deprecated, please use LOG.warning!")
    if "LOG.warn(" in logical_line:
        yield (0, msg)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import hosts_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestHostsClient(base.BaseServiceTest):
    FAKE_HOST_DATA = {
        "host": {
            "resource": {
                "cpu": 1,
                "disk_gb": 1028,
                "host": "c1a7de0ac9d94e4baceae031d05caae3",
                "memory_mb": 8192,
                "project": "(total)"
                }
        },
        "hosts": {
            "host_name": "c1a7de0ac9d94e4baceae031d05caae3",
            "service": "conductor",
            "zone": "internal"
        },
        "enable_hosts": {
            "host": "65c5d5b7e3bd44308e67fc50f362aee6",
            "maintenance_mode": "off_maintenance",
            "status": "enabled"
        }
        }

    FAKE_CONTROL_DATA = {
        "shutdown": {
            "host": "c1a7de0ac9d94e4baceae031d05caae3",
            "power_action": "shutdown"
        },
        "startup": {
            "host": "c1a7de0ac9d94e4baceae031d05caae3",
            "power_action": "startup"
        },
        "reboot": {
            "host": "c1a7de0ac9d94e4baceae031d05caae3",
            "power_action": "reboot"
        }}

    HOST_DATA = {'host': [FAKE_HOST_DATA['host']]}
    HOSTS_DATA = {'hosts': [FAKE_HOST_DATA['hosts']]}
    ENABLE_HOST_DATA = FAKE_HOST_DATA['enable_hosts']
    HOST_ID = "c1a7de0ac9d94e4baceae031d05caae3"
    TEST_HOST_DATA = {
        "status": "enable",
        "maintenance_mode": "disable"
    }

    def setUp(self):
        super(TestHostsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = hosts_client.HostsClient(fake_auth, 'compute',
                                               'regionOne')
        self.params = {'hostname': self.HOST_ID}
        self.func2mock = {
            'get': 'tempest.lib.common.rest_client.RestClient.get',
            'put': 'tempest.lib.common.rest_client.RestClient.put'}

    def _test_host_data(self, test_type='list', bytes_body=False):
        expected_resp = self.HOST_DATA
        if test_type != 'list':
            function_call = self.client.show_host
        else:
            expected_resp = self.HOSTS_DATA
            function_call = self.client.list_hosts
            self.params = {'host_name': self.HOST_ID}

        self.check_service_client_function(
            function_call, self.func2mock['get'],
            expected_resp, bytes_body,
            200, **self.params)

    def _test_update_hosts(self, bytes_body=False):
        expected_resp = self.ENABLE_HOST_DATA
        self.check_service_client_function(
            self.client.update_host, self.func2mock['put'],
            expected_resp, bytes_body,
            200, **self.params)

    def _test_control_host(self, control_op='reboot', bytes_body=False):
        if control_op == 'start':
            expected_resp = self.FAKE_CONTROL_DATA['startup']
            function_call = self.client.startup_host
        elif control_op == 'stop':
            expected_resp = self.FAKE_CONTROL_DATA['shutdown']
            function_call = self.client.shutdown_host
        else:
            expected_resp = self.FAKE_CONTROL_DATA['reboot']
            function_call = self.client.reboot_host

        self.check_service_client_function(
            function_call, self.func2mock['get'],
            expected_resp, bytes_body,
            200, **self.params)

    def test_show_host_with_str_body(self):
        self._test_host_data('show')

    def test_show_host_with_bytes_body(self):
        self._test_host_data('show', True)

    def test_list_host_with_str_body(self):
        self._test_host_data()

    def test_list_host_with_bytes_body(self):
        self._test_host_data(bytes_body=True)

    def test_start_host_with_str_body(self):
        self._test_control_host('start')

    def test_start_host_with_bytes_body(self):
        self._test_control_host('start', True)

    def test_stop_host_with_str_body(self):
        self._test_control_host('stop')

    def test_stop_host_with_bytes_body(self):
        self._test_control_host('stop', True)

    def test_reboot_host_with_str_body(self):
        self._test_control_host('reboot')

    def test_reboot_host_with_bytes_body(self):
        self._test_control_host('reboot', True)

    def test_update_host_with_str_body(self):
        self._test_update_hosts()

    def test_update_host_with_bytes_body(self):
        self._test_update_hosts(True)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class SnapshotManageAdminTest(base.BaseVolumeAdminTest):
    """Unmanage & manage snapshots

     This feature provides the ability to import/export volume snapshot
     from one Cinder to another and to import snapshots that have not been
     managed by Cinder from a storage back end to Cinder
    """

    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(SnapshotManageAdminTest, cls).skip_checks()

        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder volume snapshots are disabled")

        if not CONF.volume_feature_enabled.manage_snapshot:
            raise cls.skipException("Manage snapshot tests are disabled")

        if len(CONF.volume.manage_snapshot_ref) != 2:
            msg = ("Manage snapshot ref is not correctly configured, "
                   "it should be a list of two elements")
            raise exceptions.InvalidConfiguration(msg)

    def _test_unmanage_manage_snapshot(self, attached_volume=False):
        """Test unmanaging and managing volume snapshot"""
        # Create a volume
        volume = self.create_volume()

        # Create a snapshot
        snapshot = self.create_snapshot(volume_id=volume['id'])

        if attached_volume:
            # Create a server
            server = self.create_server(wait_until='SSHABLE')
            # Attach volume to instance
            self.attach_volume(server['id'], volume['id'],
                               wait_for_detach=False)

        # Unmanage the snapshot
        # Unmanage snapshot function works almost the same as delete snapshot,
        # but it does not delete the snapshot data
        self.admin_snapshots_client.unmanage_snapshot(snapshot['id'])
        self.admin_snapshots_client.wait_for_resource_deletion(snapshot['id'])

        # Verify the original snapshot does not exist in snapshot list
        params = {'all_tenants': 1}
        all_snapshots = self.admin_snapshots_client.list_snapshots(
            detail=True, **params)['snapshots']
        self.assertNotIn(snapshot['id'], [v['id'] for v in all_snapshots])

        # Manage the snapshot
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Managed-Snapshot')
        description = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Managed-Snapshot-Description')
        metadata = {"manage-snap-meta1": "value1",
                    "manage-snap-meta2": "value2",
                    "manage-snap-meta3": "value3"}
        snapshot_ref = {
            'volume_id': volume['id'],
            'ref': {CONF.volume.manage_snapshot_ref[0]:
                    CONF.volume.manage_snapshot_ref[1] % snapshot['id']},
            'name': name,
            'description': description,
            'metadata': metadata
        }
        new_snapshot = self.admin_snapshot_manage_client.manage_snapshot(
            **snapshot_ref)['snapshot']
        self.addCleanup(self.delete_snapshot, new_snapshot['id'],
                        self.admin_snapshots_client)

        # Wait for the snapshot to be available after manage operation
        waiters.wait_for_volume_resource_status(self.admin_snapshots_client,
                                                new_snapshot['id'],
                                                'available')

        # Verify the managed snapshot has the expected parent volume
        # and the expected field values.
        new_snapshot_info = self.admin_snapshots_client.show_snapshot(
            new_snapshot['id'])['snapshot']
        self.assertEqual(snapshot['size'], new_snapshot_info['size'])
        for key in ['volume_id', 'name', 'description', 'metadata']:
            self.assertEqual(snapshot_ref[key], new_snapshot_info[key])

    @decorators.idempotent_id('0132f42d-0147-4b45-8501-cc504bbf7810')
    def test_unmanage_manage_snapshot(self):
        self._test_unmanage_manage_snapshot()

    @decorators.idempotent_id('7c735385-e953-4198-8534-68137f72dbdc')
    @utils.services('compute')
    def test_snapshot_manage_with_attached_volume(self):
        """Test manage a snapshot with an attached volume.

           The case validates manage snapshot operation while
           the parent volume is attached to an instance.
        """
        self._test_unmanage_manage_snapshot(attached_volume=True)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures

from tempest.lib.services.identity.v3 import oauth_token_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib import fake_http
from tempest.tests.lib.services import base


class TestOAUTHTokenClient(base.BaseServiceTest):
    FAKE_CREATE_REQUEST_TOKEN = {
        'oauth_token': '29971f',
        'oauth_token_secret': '238eb8',
        'oauth_expires_at': '2013-09-11T06:07:51.501805Z'
    }

    FAKE_AUTHORIZE_REQUEST_TOKEN = {
        'token': {
            'oauth_verifier': '8171'
        }
    }

    FAKE_CREATE_ACCESS_TOKEN = {
        'oauth_token': 'accd36',
        'oauth_token_secret': 'aa47da',
        'oauth_expires_at': '2013-09-11T06:07:51.501805Z'
    }

    FAKE_ACCESS_TOKEN_INFO = {
        'access_token': {
            'consumer_id': '7fea2d',
            'id': '6be26a',
            'expires_at': '2013-09-11T06:07:51.501805Z',
            'links': {
                'roles': 'http://example.com/identity/v3/' +
                         'users/ce9e07/OS-OAUTH1/access_tokens/6be26a/roles',
                'self': 'http://example.com/identity/v3/' +
                        'users/ce9e07/OS-OAUTH1/access_tokens/6be26a'
            },
            'project_id': 'b9fca3',
            'authorizing_user_id': 'ce9e07'
        }
    }

    FAKE_LIST_ACCESS_TOKENS = {
        'access_tokens': [
            {
                'consumer_id': '7fea2d',
                'id': '6be26a',
                'expires_at': '2013-09-11T06:07:51.501805Z',
                'links': {
                    'roles': 'http://example.com/identity/v3/' +
                             'users/ce9e07/OS-OAUTH1/access_tokens/' +
                             '6be26a/roles',
                    'self': 'http://example.com/identity/v3/' +
                            'users/ce9e07/OS-OAUTH1/access_tokens/6be26a'
                },
                'project_id': 'b9fca3',
                'authorizing_user_id': 'ce9e07'
            }
        ],
        'links': {
            'next': None,
            'previous': None,
            'self': 'http://example.com/identity/v3/' +
                    'users/ce9e07/OS-OAUTH1/access_tokens'
        }
    }

    FAKE_LIST_ACCESS_TOKEN_ROLES = {
        'roles': [
            {
                'id': '26b860',
                'domain_id': 'fake_domain',
                'links': {
                    'self': 'http://example.com/identity/v3/' +
                            'roles/26b860'
                },
                'name': 'fake_role'
            }
        ],
        'links': {
            'next': None,
            'previous': None,
            'self': 'http://example.com/identity/v3/' +
                    'users/ce9e07/OS-OAUTH1/access_tokens/6be26a/roles'
        }
    }

    FAKE_ACCESS_TOKEN_ROLE_INFO = {
        'role': {
            'id': '26b860',
            'domain_id': 'fake_domain',
            'links': {
                'self': 'http://example.com/identity/v3/' +
                        'roles/26b860'
            },
            'name': 'fake_role'
        }
    }

    def setUp(self):
        super(TestOAUTHTokenClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = oauth_token_client.OAUTHTokenClient(fake_auth,
                                                          'identity',
                                                          'regionOne')

    def _mock_token_response(self, body):
        temp_response = [key + '=' + value for key, value in body.items()]
        return '&'.join(temp_response)

    def _test_authorize_request_token(self, bytes_body=False):
        self.check_service_client_function(
            self.client.authorize_request_token,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_AUTHORIZE_REQUEST_TOKEN,
            bytes_body,
            request_token_id=self.FAKE_CREATE_REQUEST_TOKEN['oauth_token'],
            role_ids=['26b860'],
            status=200)

    def test_create_request_token(self):
        mock_resp = self._mock_token_response(self.FAKE_CREATE_REQUEST_TOKEN)
        resp = fake_http.fake_http_response(None, status=201), mock_resp
        self.useFixture(fixtures.MockPatch(
            'tempest.lib.common.rest_client.RestClient.post',
            return_value=resp))

        resp = self.client.create_request_token(
            consumer_key='12345',
            consumer_secret='23456',
            project_id='c8f58432c6f00162f04d3250f')
        self.assertEqual(self.FAKE_CREATE_REQUEST_TOKEN, resp)

    def test_authorize_token_request_with_str_body(self):
        self._test_authorize_request_token()

    def test_authorize_token_request_with_bytes_body(self):
        self._test_authorize_request_token(bytes_body=True)

    def test_create_access_token(self):
        mock_resp = self._mock_token_response(self.FAKE_CREATE_ACCESS_TOKEN)
        req_secret = self.FAKE_CREATE_REQUEST_TOKEN['oauth_token_secret']
        resp = fake_http.fake_http_response(None, status=201), mock_resp
        self.useFixture(fixtures.MockPatch(
            'tempest.lib.common.rest_client.RestClient.post',
            return_value=resp))

        resp = self.client.create_access_token(
            consumer_key='12345',
            consumer_secret='23456',
            request_key=self.FAKE_CREATE_REQUEST_TOKEN['oauth_token'],
            request_secret=req_secret,
            oauth_verifier='8171')
        self.assertEqual(self.FAKE_CREATE_ACCESS_TOKEN, resp)

    def test_get_access_token(self):
        self.check_service_client_function(
            self.client.get_access_token,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ACCESS_TOKEN_INFO,
            user_id='ce9e07',
            access_token_id=self.FAKE_ACCESS_TOKEN_INFO['access_token']['id'],
            status=200)

    def test_list_access_tokens(self):
        self.check_service_client_function(
            self.client.list_access_tokens,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ACCESS_TOKENS,
            user_id='ce9e07',
            status=200)

    def test_revoke_access_token(self):
        self.check_service_client_function(
            self.client.revoke_access_token,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            user_id=self.FAKE_ACCESS_TOKEN_INFO['access_token']['consumer_id'],
            access_token_id=self.FAKE_ACCESS_TOKEN_INFO['access_token']['id'],
            status=204)

    def test_list_access_token_roles(self):
        self.check_service_client_function(
            self.client.list_access_token_roles,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ACCESS_TOKEN_ROLES,
            user_id='ce9e07',
            access_token_id=self.FAKE_ACCESS_TOKEN_INFO['access_token']['id'],
            status=200)

    def test_get_access_token_role(self):
        self.check_service_client_function(
            self.client.get_access_token_role,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ACCESS_TOKEN_ROLE_INFO,
            user_id='ce9e07',
            access_token_id=self.FAKE_ACCESS_TOKEN_INFO['access_token']['id'],
            role_id=self.FAKE_ACCESS_TOKEN_ROLE_INFO['role']['id'],
            status=200)



# Copyright 2014 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from tempest.lib import exceptions


# Define the minimum and maximum version of the API across all of the
# REST API. The format of the version is:
# X.Y where:
#
# - X will only be changed if a significant backwards incompatible API
# change is made which affects the API as whole. That is, something
# that is only very very rarely incremented.
#
# - Y when you make any change to the API. Note that this includes
# semantic changes which may not affect the input or output formats or
# even originate in the API code layer. We are not distinguishing
# between backwards compatible and backwards incompatible changes in
# the versioning system. It must be made clear in the documentation as
# to what is a backwards compatible change and what is a backwards
# incompatible one.

class APIVersionRequest(object):
    """This class represents an API Version Request.

    This class provides convenience methods for manipulation
    and comparison of version numbers that we need to do to
    implement microversions.

    :param version_string: String representation of APIVersionRequest.
            Correct format is 'X.Y', where 'X' and 'Y' are int values.
            None value should be used to create Null APIVersionRequest,
            which is equal to 0.0
    """

    # NOTE: This 'latest' version is a magic number, we assume any
    # projects(Nova, etc.) never achieve this number.
    latest_ver_major = 99999
    latest_ver_minor = 99999

    def __init__(self, version_string=None):
        """Create an API version request object."""
        # NOTE(gmann): 'version_string' as String "None" will be considered as
        # invalid version string.
        self.ver_major = 0
        self.ver_minor = 0

        if version_string is not None:
            match = re.match(r"^([1-9]\d*)\.([1-9]\d*|0)$",
                             version_string)
            if match:
                self.ver_major = int(match.group(1))
                self.ver_minor = int(match.group(2))
            elif version_string == 'latest':
                self.ver_major = self.latest_ver_major
                self.ver_minor = self.latest_ver_minor
            else:
                raise exceptions.InvalidAPIVersionString(
                    version=version_string)

    def __str__(self):
        """Debug/Logging representation of object."""
        return ("API Version Request: %s" % self.get_string())

    def is_null(self):
        """Checks whether version is null.

        Return True if version object is null otherwise False.

        :returns: boolean
        """
        return self.ver_major == 0 and self.ver_minor == 0

    def _format_type_error(self, other):
        return TypeError("'%(other)s' should be an instance of '%(cls)s'" %
                         {"other": other, "cls": self.__class__})

    def __lt__(self, other):
        if not isinstance(other, APIVersionRequest):
            raise self._format_type_error(other)

        return ((self.ver_major, self.ver_minor) <
                (other.ver_major, other.ver_minor))

    def __eq__(self, other):
        if not isinstance(other, APIVersionRequest):
            raise self._format_type_error(other)

        return ((self.ver_major, self.ver_minor) ==
                (other.ver_major, other.ver_minor))

    def __gt__(self, other):
        if not isinstance(other, APIVersionRequest):
            raise self._format_type_error(other)

        return ((self.ver_major, self.ver_minor) >
                (other.ver_major, other.ver_minor))

    def __le__(self, other):
        return self < other or self == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __ge__(self, other):
        return self > other or self == other

    def matches(self, min_version, max_version):
        """Matches the version object.

        Returns whether the version object represents a version
        greater than or equal to the minimum version and less than
        or equal to the maximum version.

        :param min_version: Minimum acceptable version.
        :param max_version: Maximum acceptable version.
        :returns: boolean

        If min_version is null then there is no minimum limit.
        If max_version is null then there is no maximum limit.
        If self is null then raise ValueError
        """

        if self.is_null():
            raise ValueError
        if max_version.is_null() and min_version.is_null():
            return True
        elif max_version.is_null():
            return min_version <= self
        elif min_version.is_null():
            return self <= max_version
        else:
            return min_version <= self <= max_version

    def get_string(self):
        """Version string representation.

        Converts object to string representation which if used to create
        an APIVersionRequest object results in the same version request.
        """
        if self.is_null():
            return None
        if (self.ver_major == self.latest_ver_major and
            self.ver_minor == self.latest_ver_minor):
            return 'latest'
        return "%s.%s" % (self.ver_major, self.ver_minor)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
from tempest.scenario import manager

CONF = config.CONF


class TestServerMultinode(manager.ScenarioTest):
    """This is a set of tests specific to multinode testing."""
    credentials = ['primary', 'admin']

    @classmethod
    def skip_checks(cls):
        super(TestServerMultinode, cls).skip_checks()

        if CONF.compute.min_compute_nodes < 2:
            raise cls.skipException(
                "Less than 2 compute nodes, skipping multinode tests.")

    @decorators.idempotent_id('9cecbe35-b9d4-48da-a37e-7ce70aa43d30')
    @decorators.attr(type=['smoke', 'multinode'])
    @utils.services('compute', 'network')
    def test_schedule_to_all_nodes(self):
        available_zone = \
            self.os_admin.availability_zone_client.list_availability_zones(
                detail=True)['availabilityZoneInfo']
        hosts = []
        for zone in available_zone:
            if zone['zoneState']['available']:
                for host in zone['hosts']:
                    if 'nova-compute' in zone['hosts'][host] and \
                        zone['hosts'][host]['nova-compute']['available'] and \
                        CONF.compute.target_hosts_to_avoid not in host:
                        hosts.append({'zone': zone['zoneName'],
                                      'host_name': host})

        # ensure we have at least as many compute hosts as we expect
        if len(hosts) < CONF.compute.min_compute_nodes:
            raise exceptions.InvalidConfiguration(
                "Host list %s is shorter than min_compute_nodes. "
                "Did a compute worker not boot correctly?" % hosts)

        # create 1 compute for each node, up to the min_compute_nodes
        # threshold (so that things don't get crazy if you have 1000
        # compute nodes but set min to 3).
        servers = []
        host_server_ids = {}

        for host in hosts[:CONF.compute.min_compute_nodes]:
            # by getting to active state here, this means this has
            # landed on the host in question.
            # in order to use the availability_zone:host scheduler hint,
            # admin client is need here.
            inst = self.create_server(
                wait_until=None,
                clients=self.os_admin,
                availability_zone='%(zone)s:%(host_name)s' % host)
            host_server_ids[host['host_name']] = inst['id']

        for host_name, server_id in host_server_ids.items():
            waiters.wait_for_server_status(self.os_admin.servers_client,
                                           server_id, 'ACTIVE')
            server = self.os_admin.servers_client.show_server(
                server_id)['server']
            # ensure server is located on the requested host
            self.assertEqual(host_name, server['OS-EXT-SRV-ATTR:host'])
            servers.append(server)

        # make sure we really have the number of servers we think we should
        self.assertEqual(
            len(servers), CONF.compute.min_compute_nodes,
            "Incorrect number of servers built %s" % servers)

        # ensure that every server ended up on a different host
        host_ids = [x['hostId'] for x in servers]
        self.assertEqual(
            len(set(host_ids)), len(servers),
            "Incorrect number of distinct host_ids scheduled to %s" % servers)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF
QUOTA_KEYS = ['gigabytes', 'snapshots', 'volumes', 'backups',
              'backup_gigabytes', 'per_volume_gigabytes']


class VolumeQuotasNegativeTestJSON(base.BaseVolumeAdminTest):
    """Negative tests of volume quotas"""

    @classmethod
    def setup_credentials(cls):
        super(VolumeQuotasNegativeTestJSON, cls).setup_credentials()
        cls.demo_tenant_id = cls.os_primary.credentials.tenant_id

    @classmethod
    def resource_setup(cls):
        super(VolumeQuotasNegativeTestJSON, cls).resource_setup()

        # Save the current set of quotas, then set up the cleanup method
        # to restore the quotas to their original values after the tests
        # from this class are done. This is needed just in case Tempest is
        # configured to use pre-provisioned projects/user accounts.
        original_quota_set = (cls.admin_quotas_client.show_quota_set(
            cls.demo_tenant_id)['quota_set'])
        cleanup_quota_set = dict(
            (k, v) for k, v in original_quota_set.items() if k in QUOTA_KEYS)
        cls.addClassResourceCleanup(cls.admin_quotas_client.update_quota_set,
                                    cls.demo_tenant_id, **cleanup_quota_set)

        # NOTE(gfidente): no need to delete in tearDown as
        # they are created using utility wrapper methods.
        cls.volume = cls.create_volume()

    @decorators.attr(type='negative')
    @decorators.idempotent_id('bf544854-d62a-47f2-a681-90f7a47d86b6')
    def test_quota_volumes(self):
        """Creating more volume than allowed quota will fail"""
        self.admin_quotas_client.update_quota_set(self.demo_tenant_id,
                                                  volumes=1, gigabytes=-1)
        self.assertRaises(lib_exc.OverLimit,
                          self.volumes_client.create_volume,
                          size=CONF.volume.volume_size)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('2dc27eee-8659-4298-b900-169d71a91374')
    def test_quota_volume_gigabytes(self):
        """Creating volume with size larger than allowed quota will fail"""
        self.admin_quotas_client.update_quota_set(
            self.demo_tenant_id, gigabytes=CONF.volume.volume_size, volumes=-1)
        self.assertRaises(lib_exc.OverLimit,
                          self.volumes_client.create_volume,
                          size=CONF.volume.volume_size * 2)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d321dc21-d8c6-401f-95fe-49f4845f1a6d')
    def test_volume_extend_gigabytes_quota_deviation(self):
        """Extending volume with size larger than allowed quota will fail"""
        self.admin_quotas_client.update_quota_set(
            self.demo_tenant_id, gigabytes=CONF.volume.volume_size)
        self.assertRaises(lib_exc.OverLimit,
                          self.volumes_client.extend_volume,
                          self.volume['id'],
                          new_size=CONF.volume.volume_size * 2)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import tempest_fixtures as fixtures
from tempest.lib import decorators


class QuotasTestJSON(base.BaseV2ComputeTest):
    """Test compute quotas"""

    def setUp(self):
        # NOTE(mriedem): Avoid conflicts with os-quota-class-sets tests.
        self.useFixture(fixtures.LockFixture('compute_quotas'))
        super(QuotasTestJSON, self).setUp()

    @classmethod
    def setup_clients(cls):
        super(QuotasTestJSON, cls).setup_clients()
        cls.client = cls.quotas_client

    @classmethod
    def resource_setup(cls):
        super(QuotasTestJSON, cls).resource_setup()
        cls.tenant_id = cls.client.tenant_id
        cls.user_id = cls.client.user_id
        cls.default_quota_set = set(('metadata_items', 'ram', 'key_pairs',
                                     'instances', 'cores',
                                     'server_group_members', 'server_groups'))
        if cls.is_requested_microversion_compatible('2.35'):
            cls.default_quota_set = \
                cls.default_quota_set | set(['fixed_ips', 'floating_ips',
                                             'security_group_rules',
                                             'security_groups'])
        if cls.is_requested_microversion_compatible('2.56'):
            cls.default_quota_set = \
                cls.default_quota_set | set(['injected_file_content_bytes',
                                             'injected_file_path_bytes',
                                             'injected_files'])

    @decorators.idempotent_id('f1ef0a97-dbbb-4cca-adc5-c9fbc4f76107')
    def test_get_quotas(self):
        """Test user can get the compute quota set for it's project"""
        expected_quota_set = self.default_quota_set | set(['id'])
        quota_set = self.client.show_quota_set(self.tenant_id)['quota_set']
        self.assertEqual(quota_set['id'], self.tenant_id)
        for quota in expected_quota_set:
            self.assertIn(quota, quota_set.keys())

        # get the quota set using user id
        quota_set = self.client.show_quota_set(
            self.tenant_id, user_id=self.user_id)['quota_set']
        self.assertEqual(quota_set['id'], self.tenant_id)
        for quota in expected_quota_set:
            self.assertIn(quota, quota_set.keys())

    @decorators.idempotent_id('9bfecac7-b966-4f47-913f-1a9e2c12134a')
    def test_get_default_quotas(self):
        """Test user can get the default compute quota set for it's project"""
        expected_quota_set = self.default_quota_set | set(['id'])
        quota_set = (self.client.show_default_quota_set(self.tenant_id)
                     ['quota_set'])
        self.assertEqual(quota_set['id'], self.tenant_id)
        for quota in expected_quota_set:
            self.assertIn(quota, quota_set.keys())

    @decorators.idempotent_id('cd65d997-f7e4-4966-a7e9-d5001b674fdc')
    def test_compare_tenant_quotas_with_default_quotas(self):
        """Test tenants are created with the default compute quota values"""
        default_quota_set = \
            self.client.show_default_quota_set(self.tenant_id)['quota_set']
        tenant_quota_set = (self.client.show_quota_set(self.tenant_id)
                            ['quota_set'])
        self.assertEqual(default_quota_set, tenant_quota_set)



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.network import base


class QosLimitBandwidthRulesClient(base.BaseNetworkClient):

    def create_limit_bandwidth_rule(self, qos_policy_id, **kwargs):
        """Creates a limit bandwidth rule for a QoS policy.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-bandwidth-limit-rule
        """
        uri = '/qos/policies/{}/bandwidth_limit_rules'.format(
            qos_policy_id)
        post_data = {'bandwidth_limit_rule': kwargs}
        return self.create_resource(uri, post_data)

    def update_limit_bandwidth_rule(self, qos_policy_id, rule_id, **kwargs):
        """Updates a limit bandwidth rule.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-bandwidth-limit-rule
        """
        uri = '/qos/policies/{}/bandwidth_limit_rules/{}'.format(
            qos_policy_id, rule_id)
        post_data = {'bandwidth_limit_rule': kwargs}
        return self.update_resource(uri, post_data)

    def show_limit_bandwidth_rule(self, qos_policy_id, rule_id, **fields):
        """Show details of a limit bandwidth rule.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-bandwidth-limit-rule-details
        """
        uri = '/qos/policies/{}/bandwidth_limit_rules/{}'.format(
            qos_policy_id, rule_id)
        return self.show_resource(uri, **fields)

    def delete_limit_bandwidth_rule(self, qos_policy_id, rule_id):
        """Deletes a limit bandwidth rule for a QoS policy.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#delete-bandwidth-limit-rule
        """
        uri = '/qos/policies/{}/bandwidth_limit_rules/{}'.format(
            qos_policy_id, rule_id)
        return self.delete_resource(uri)

    def list_limit_bandwidth_rules(self, qos_policy_id, **filters):
        """Lists all limit bandwidth rules for a QoS policy.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-bandwidth-limit-rules-for-qos-policy
        """
        uri = '/qos/policies/{}/bandwidth_limit_rules'.format(qos_policy_id)
        return self.list_resources(uri, **filters)
